//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include <ComCtrls.hpp>
#include <Dialogs.hpp>
#include <IniFiles.hpp>
#include "App_Documents.h"
#include "Moa_AutoPointers.h"
#include "Moa_Clipboard.h"
#include "Moa_DynamicArrays.h"
#include "Moa_Files.h"
#include "Moa_ListViews.h"
#include "Moa_ShellFiles.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_documents {
//---------------------------------------------------------------------------
void __fastcall TDocumentEntry::Initialize(const AnsiString& AFile)
{
	try {
		{
			TPathUtilities::AssureAbsolute(AFile);
			FPath = TFileUtilities::GetLongPath(ExpandFileName(AFile));
			FDirectory = ExtractFileDir(FPath);
			FName = ExtractFileName(FPath);
		}
		{
			static TFileTypeCache* LFileTypeCache = NULL;
			if(LFileTypeCache == NULL) {
				LFileTypeCache = new TFileTypeCache();
			}
			FType = LFileTypeCache->Determine(FPath);
		}
		{
			WIN32_FIND_DATA LFileData = TFileUtilities::GetData(FPath);
			FSize = TFileDataUtilities::GetSize(LFileData);
			FLastWriteTime = TFileDataUtilities::GetLastWriteTime(LFileData);
			FAttributes = TFileDataUtilities::GetAttributes(LFileData);
		}
	}
	catch(EFileError& LException) {
		throw EDocumentEntryCreateError(AnsiString().sprintf(
			"The program could not create a document entry with the path \"%s\".",
			AFile.c_str()));
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntry::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocumentEntry::TDocumentEntry(const AnsiString& AFile) :
	TObject(),
	FPath(),
	FDirectory(),
	FName(),
	FType(),
	FSize(0),
	FLastWriteTime(),
	FAttributes(0),
	FResult(orNone)
{
	Initialize(AFile);
}
//---------------------------------------------------------------------------
__fastcall TDocumentEntry::~TDocumentEntry()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntry::IsRegularFile()
{
	return !IsDirectory();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntry::IsDirectory()
{
	return (Attributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int TDocumentEntryDescriber::ImageIndex_RegularFile = -1;
int TDocumentEntryDescriber::ImageIndex_Directory = -1;
int TDocumentEntryDescriber::StateIndex_None = -1;
int TDocumentEntryDescriber::StateIndex_Success = -1;
int TDocumentEntryDescriber::StateIndex_Failure = -1;
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeByPath(TDocumentEntry* ADocumentEntry)
{
	return DescribePath(ADocumentEntry->Path);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeByDirectory(TDocumentEntry* ADocumentEntry)
{
	return DescribePath(ADocumentEntry->Directory);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeByName(TDocumentEntry* ADocumentEntry)
{
	return DescribePath(ADocumentEntry->Name);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeBySize(TDocumentEntry* ADocumentEntry)
{
	if(ADocumentEntry->IsDirectory()) {
		return AnsiString();
	}
	return DescribeSize(ADocumentEntry->Size);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeByType(TDocumentEntry* ADocumentEntry)
{
	return DescribeType(ADocumentEntry->Type);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeByLastWriteTime(TDocumentEntry* ADocumentEntry)
{
	return DescribeTime(ADocumentEntry->LastWriteTime);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeByResult(TDocumentEntry* ADocumentEntry)
{
	return DescribeResult(ADocumentEntry->Result);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribePath(const AnsiString& APath)
{
	return APath;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeType(const AnsiString& AType)
{
	return AType;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeTime(const FILETIME& ATime)
{
	FILETIME LFileTime = {0};
	SYSTEMTIME LSystemTime = {0};
	if((ATime.dwLowDateTime == 0 && ATime.dwHighDateTime == 0) ||
		!::FileTimeToLocalFileTime(&ATime, &LFileTime) ||
		!::FileTimeToSystemTime(&LFileTime, &LSystemTime)) {
		return "(Unknown)";
	}
	return AnsiString().sprintf(
		"%04d-%02d-%02d %02d:%02d:%02d",
		LSystemTime.wYear,
		LSystemTime.wMonth,
		LSystemTime.wDay,
		LSystemTime.wHour,
		LSystemTime.wMinute,
		LSystemTime.wSecond);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeSize(DWORDLONG ASize)
{
	return TAnsiStringUtilities::FormatInteger(ASize, 3, ',');
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntryDescriber::DescribeResult(TOpenResult AResult)
{
	switch(AResult) {
	case orNone:
		return "None";
	case orSuccess:
		return "Success";
	case orFailure:
		return "Failure";
	default:
		throw EAppError("The program does not support the result.");
	}
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareByPath(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return ComparePath(ADocumentEntry1->Path, ADocumentEntry2->Path);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareByDirectory(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return ComparePath(ADocumentEntry1->Directory, ADocumentEntry2->Directory);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareByName(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return ComparePath(ADocumentEntry1->Name, ADocumentEntry2->Name);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareBySize(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	if(ADocumentEntry1->IsDirectory()) {
		if(ADocumentEntry2->IsDirectory()) {
			return 0;
		}
		else {
			return -1;
		}
	}
	else {
		if(ADocumentEntry2->IsDirectory()) {
			return 1;
		}
		else {
			return CompareSize(ADocumentEntry1->Size, ADocumentEntry2->Size);
		}
	}
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareByType(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return CompareType(ADocumentEntry1->Type, ADocumentEntry2->Type);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareByLastWriteTime(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return CompareTime(ADocumentEntry1->LastWriteTime, ADocumentEntry2->LastWriteTime);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareByResult(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return CompareResult(ADocumentEntry1->Result, ADocumentEntry2->Result);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::ComparePath(const AnsiString& APath1, const AnsiString& APath2)
{
	return AnsiCompareFileName(APath1, APath2);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareType(const AnsiString& AType1, const AnsiString& AType2)
{
	return AnsiCompareStr(AType1, AType2);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareTime(const FILETIME& ATime1, const FILETIME& ATime2)
{
	return ::CompareFileTime(&ATime1, &ATime2);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareSize(DWORDLONG ASize1, DWORDLONG ASize2)
{
	if(ASize1 < ASize2) {
		return -1;
	}
	else if(ASize1 > ASize2) {
		return 1;
	}
	else {
		return 0;
	}
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::CompareResult(TOpenResult AResult1, TOpenResult AResult2)
{
	return EvaluateResult(AResult1) - EvaluateResult(AResult2);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::EvaluateResult(TOpenResult AResult)
{
	switch(AResult) {
	case orSuccess:
		return 0;
	case orFailure:
		return 1;
	case orNone:
		return 2;
	default:
		throw EAppError("The program does not support the result.");
	}
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::DetermineImageIndex(TDocumentEntry* ADocumentEntry)
{
	if(ADocumentEntry->IsDirectory()) {
		return ImageIndex_Directory;
	}
	else {
		return ImageIndex_RegularFile;
	}
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntryDescriber::DetermineStateIndex(TDocumentEntry* ADocumentEntry)
{
	switch(ADocumentEntry->Result) {
	case orNone:
		return StateIndex_None;
	case orSuccess:
		return StateIndex_Success;
	case orFailure:
		return StateIndex_Failure;
	default:
		throw EAppError("The program does not support the result.");
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::SortCompareByPath(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentEntryDescriber::CompareByPath(
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::SortCompareByDirectory(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentEntryDescriber::CompareByDirectory(
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::SortCompareByName(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentEntryDescriber::CompareByName(
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::SortCompareBySize(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentEntryDescriber::CompareBySize(
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::SortCompareByType(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentEntryDescriber::CompareByType(
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::SortCompareByLastWriteTime(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentEntryDescriber::CompareByLastWriteTime(
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::SortCompareByResult(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentEntryDescriber::CompareByResult(
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentEntriesModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::Initialize()
{
	try {
		FList = new TObjectList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
TDocumentEntry* __fastcall TDocumentEntriesModel::GetItem(int AIndex)
{
	return static_cast<TDocumentEntry*>(List->Items[AIndex]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::GetCapacity()
{
	return List->Capacity;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::SetCapacity(int ACapacity)
{
	List->Capacity = ACapacity;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::DoInsertItems(const TIntegerDynArray& AIndexes)
{
	if(OnInsertItems != NULL) {
		OnInsertItems(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::DoDeleteItems(const TIntegerDynArray& AIndexes)
{
	if(OnDeleteItems != NULL) {
		OnDeleteItems(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::DoClear()
{
	if(OnClear != NULL) {
		OnClear(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::DoReorder(const TReorderRecords& ARecords)
{
	if(OnReorder != NULL) {
		OnReorder(this, ARecords);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::DoItemsChange(const TIntegerDynArray& AIndexes)
{
	if(OnItemsChange != NULL) {
		OnItemsChange(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
__fastcall TDocumentEntriesModel::TDocumentEntriesModel() :
	TModel(),
	FList(NULL),
	FOnInsertItems(NULL),
	FOnDeleteItems(NULL),
	FOnClear(NULL),
	FOnReorder(NULL),
	FOnItemsChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TDocumentEntriesModel::~TDocumentEntriesModel()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::AddItems(const DynamicArray<TDocumentEntry*>& AItems)
{
	if(AItems.Length == 0) {
		return;
	}
	TIntegerDynArray LIndexes;
	LIndexes.Length = AItems.Length;
	TListUtilities::EnsureCapacity(List, List->Count + AItems.Length);
	for(int LI = 0; LI < AItems.Length; ++LI) {
		LIndexes[LI] = List->Add(AItems[LI]);
	}
	DoInsertItems(LIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::DeleteItems(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		List->Delete(AIndexes[AIndexes.High - LI]);
	}
	DoDeleteItems(AIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::Clear()
{
	if(Count == 0) {
		return;
	}
	List->Clear();
	DoClear();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::SortByPath()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByPath, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::SortByDirectory()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByDirectory, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::SortByName()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByName, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::SortBySize()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareBySize, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::SortByType()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByType, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::SortByLastWriteTime()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByLastWriteTime, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::SortByResult()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByResult, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::ReverseOrder()
{
	Reorder(TListAlgorithms::ReverseOrder(Count));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::MoveUp(const TIntegerDynArray& AIndexes)
{
	Reorder(TListAlgorithms::Slide(Count, AIndexes, -1));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::MoveDown(const TIntegerDynArray& AIndexes)
{
	Reorder(TListAlgorithms::Slide(Count, AIndexes, 1));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::Reorder(const TReorderRecords& ARecords)
{
	TObjectListUtilities::Reorder(List, ARecords);
	DoReorder(ARecords);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesModel::ItemsChange(const TIntegerDynArray& AIndexes)
{
	DoItemsChange(AIndexes);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::Find(TDocumentEntry* AItem)
{
	return List->IndexOf(AItem);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::FindByResult(TOpenResult AResult)
{
	for(int LI = 0; LI < Count; ++LI) {
		if(Items[LI]->Result == AResult) {
			return LI;
		}
	}
	return -1;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TDocumentEntriesModel::FindItemsByResult(TOpenResult AResult)
{
	TAutoPointer<TDynamicArrayBuilder<int> > LBuilder = new TDynamicArrayBuilder<int>();
	LBuilder->Capacity = CountByResult(AResult);
	for(int LI = 0; LI < Count; ++LI) {
		if(Items[LI]->Result == AResult) {
			LBuilder->Add(LI);
		}
	}
	return LBuilder->Finish();
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::CountByResult(TOpenResult AResult)
{
	int LCount = 0;
	for(int LI = 0; LI < Count; ++LI) {
		if(Items[LI]->Result == AResult) {
			++LCount;
		}
	}
	return LCount;
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesModel::CountByResultIn(TOpenResult AResult, const TIntegerDynArray& AIndexes)
{
	int LCount = 0;
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		if(Items[AIndexes[LI]]->Result == AResult) {
			++LCount;
		}
	}
	return LCount;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesSelection::GetItem(int AIndex)
{
	return TListViewItemUtilities::IsSelected(ListView, AIndex);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesSelection::SetItem(int AIndex, bool AItem)
{
	TListViewItemUtilities::SetSelected(ListView, AIndex, AItem);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesSelection::GetCount()
{
	return ListView->Items->Count;
}
//---------------------------------------------------------------------------
int __fastcall TDocumentEntriesSelection::GetSelectionCount()
{
	return ListView->SelCount;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TDocumentEntriesSelection::GetSelectionIndexes()
{
	return TListViewItemsUtilities::GetSelectionIndexes(ListView);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesSelection::SetSelectionIndexes(const TIntegerDynArray& ASelectionIndexes)
{
	TListViewItemsUtilities::SetSelectionIndexes(ListView, ASelectionIndexes);
}
//---------------------------------------------------------------------------
__fastcall TDocumentEntriesSelection::TDocumentEntriesSelection(TDocumentEntriesModel* ADocumentEntriesModel, TListView* AListView) :
	TSelection(),
	FDocumentEntriesModel(ADocumentEntriesModel),
	FListView(AListView)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocumentEntriesSelection::~TDocumentEntriesSelection()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesSelection::HandleInsertItems(TObject* ASender, const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::InsertItems(ListView, AIndexes);
		TListViewItemsUtilities::SetSelectionIndexes(ListView, AIndexes);
		TListViewItemUtilities::SetFocused(ListView, AIndexes[AIndexes.High], true);
		TListViewItemUtilities::EnsureVisible(ListView, AIndexes[AIndexes.High], false);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesSelection::HandleDeleteItems(TObject* ASender, const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::DeleteItems(ListView, AIndexes);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesSelection::HandleClear(TObject* ASender)
{
	if(ListView->Items->Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		ListView->Items->Count = 0;
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesSelection::HandleReorder(TObject* ASender, const TReorderRecords& ARecords)
{
	if(ARecords.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::Reorder(ListView, ARecords);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesSelection::HandleItemsChange(TObject* ASender, const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	try {
		for(int LI = 0; LI < AIndexes.Length; ++LI) {
			ListView->UpdateItems(AIndexes[LI], AIndexes[LI]);
		}
	}
	catch(...) {
		ListView->Update();
		throw;
	}
	ListView->Update();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesSelection::SelectAll()
{
	if(Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		ListView->SelectAll();
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesSelection::ClearSelection()
{
	if(Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		ListView->ClearSelection();
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesSelection::ReverseSelection()
{
	if(Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
 		TListViewItemsUtilities::ReverseSelection(ListView);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::Initialize()
{
	try {
		FClipboardWatcher = new TClipboardWatcher();
		FClipboardWatcher->Enabled = true;
		FClipboardWatcher->OnChange = ClipboardWatcherChange;
		FAddRegularFilesDialog = new TOpenDialog(NULL);
		FAddRegularFilesDialog->Title = "Add Regular Files";
		FAddRegularFilesDialog->Filter = "All (*.*)|*.*";
		FAddRegularFilesDialog->FilterIndex = 1;
		FAddRegularFilesDialog->Options = FAddRegularFilesDialog->Options
			<< ofAllowMultiSelect
			<< ofFileMustExist;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::Uninitialize()
{
	delete FAddRegularFilesDialog;
	FAddRegularFilesDialog = NULL;

	delete FClipboardWatcher;
	FClipboardWatcher = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::ClipboardWatcherChange(TObject* ASender)
{
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TDocumentEntriesCommands::TDocumentEntriesCommands(TDocumentEntriesModel* ADocumentEntriesModel, TDocumentEntriesSelection* ADocumentEntriesSelection) :
	TCommands(),
	FDocumentEntriesModel(ADocumentEntriesModel),
	FDocumentEntriesSelection(ADocumentEntriesSelection),
	FClipboardWatcher(NULL),
	FAddRegularFilesDialog(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TDocumentEntriesCommands::~TDocumentEntriesCommands()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::New()
{
	if(!CanNew()) {
		HandleFeatureUnavailable();
	}
	if(DocumentEntriesModel->CountByResult(orSuccess) != DocumentEntriesModel->Count) {
		if(Application->MessageBox(
			"Do you want to remove all the documents from the list?",
			Application->Title.c_str(),
			MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) != IDYES) {
			Abort();
		}
	}
	DocumentEntriesModel->Clear();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanNew()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::AddRegularFiles()
{
	if(!CanAddRegularFiles()) {
		HandleFeatureUnavailable();
	}
	AddRegularFilesDialog->FileName = AnsiString();
	if(!AddRegularFilesDialog->Execute()) {
		Abort();
	}
	AddFiles(TAnsiStringDynamicArrayUtilities::FromList(AddRegularFilesDialog->Files));
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanAddRegularFiles()
{
	return CanAddFiles();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::AddDirectory()
{
	if(!CanAddDirectory()) {
		HandleFeatureUnavailable();
	}
	AnsiString LDirectory = GetCurrentDir();
	if(!SelectDirectory("Specify a directory.", WideString(), LDirectory)) {
		Abort();
	}
	SetCurrentDir(LDirectory);
	AddFiles(TAnsiStringDynamicArrayUtilities::FromElements(LDirectory));
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanAddDirectory()
{
	return CanAddFiles();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::AddDroppedFiles(const TStringDynArray& ADroppedFiles)
{
	AddFiles(ADroppedFiles);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanAddDroppedFiles()
{
	return CanAddFiles();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::AddFiles(const TStringDynArray& AFiles)
{
	if(!CanAddFiles()) {
		HandleFeatureUnavailable();
	}
	if(AFiles.Length == 0) {
		return;
	}
	TAutoPointer<TObjectList> LList = new TObjectList();
	LList->Capacity = AFiles.Length;
	for(int LI = 0; LI < AFiles.Length; ++LI) {
		TAutoPointer<TDocumentEntry> LItem = new TDocumentEntry(AFiles[LI]);
		LList->Add(LItem);
		LItem.Release();
	}
	DynamicArray<TDocumentEntry*> LItems;
	LItems.Length = LList->Count;
	for(int LI = 0; LI < LItems.Length; ++LI) {
		LItems[LI] = static_cast<TDocumentEntry*>(LList->Items[LI]);
	}
	DocumentEntriesModel->AddItems(LItems);
	LList->OwnsObjects = false;
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanAddFiles()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::Cut()
{
	if(!CanCut()) {
		HandleFeatureUnavailable();
	}
	Copy();
	Delete(false);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanCut()
{
	return CanCopy() && CanDelete();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::Copy()
{
	if(!CanCopy()) {
		HandleFeatureUnavailable();
	}
	TAutoPointer<TStrings> LLines = new TStringList();
	LLines->Capacity = DocumentEntriesSelection->SelectionCount;
	for(int LI = 0; LI < DocumentEntriesModel->Count; ++LI) {
		if(!DocumentEntriesSelection->Items[LI]) {
			continue;
		}
		LLines->Add(DocumentEntriesModel->Items[LI]->Path);
		if(LLines->Count == LLines->Capacity) {
			break;
		}
	}
	Clipboard()->AsText = LLines->Text;
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanCopy()
{
	return DocumentEntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::Paste()
{
	if(!CanPaste()) {
		HandleFeatureUnavailable();
	}
	TStringDynArray LFiles;
	LFiles = TAnsiStringDynamicArrayUtilities::FromText(Clipboard()->AsText);
	LFiles = TAnsiStringDynamicArrayUtilities::RemoveBlankLines(LFiles);
	AddFiles(LFiles);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanPaste()
{
	return CanAddFiles() && Clipboard()->HasFormat(CF_TEXT);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::Delete()
{
	if(!CanDelete()) {
		HandleFeatureUnavailable();
	}
	Delete(true);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::Delete(bool AConfirm)
{
	if(!CanDelete()) {
		HandleFeatureUnavailable();
	}
	TIntegerDynArray LIndexes = DocumentEntriesSelection->SelectionIndexes;
	if(AConfirm && DocumentEntriesModel->CountByResultIn(orSuccess, LIndexes) != LIndexes.Length) {
		if(Application->MessageBox(
			"Do you want to remove the documents from the list?",
			Application->Title.c_str(),
			MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
			Abort();
		}
	}
	DocumentEntriesModel->DeleteItems(LIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanDelete()
{
	return DocumentEntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::SelectAll()
{
	if(!CanSelectAll()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesSelection->SelectAll();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanSelectAll()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::ReverseSelection()
{
	if(!CanReverseSelection()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesSelection->ReverseSelection();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanReverseSelection()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::SortByPath()
{
	if(!CanSortByPath()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->SortByPath();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanSortByPath()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::SortByDirectory()
{
	if(!CanSortByDirectory()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->SortByDirectory();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanSortByDirectory()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::SortByName()
{
	if(!CanSortByName()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->SortByName();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanSortByName()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::SortBySize()
{
	if(!CanSortBySize()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->SortBySize();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanSortBySize()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::SortByType()
{
	if(!CanSortByType()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->SortByType();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanSortByType()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::SortByLastWriteTime()
{
	if(!CanSortByLastWriteTime()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->SortByLastWriteTime();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanSortByLastWriteTime()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::SortByResult()
{
	if(!CanSortByResult()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->SortByResult();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanSortByResult()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::ReverseOrder()
{
	if(!CanReverseOrder()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->ReverseOrder();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanReverseOrder()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::MoveUp()
{
	if(!CanMoveUp()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->MoveUp(DocumentEntriesSelection->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanMoveUp()
{
	return DocumentEntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::MoveDown()
{
	if(!CanMoveDown()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->MoveDown(DocumentEntriesSelection->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanMoveDown()
{
	return DocumentEntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::DeleteAllInSuccess()
{
	if(!CanDeleteAllInSuccess()) {
		HandleFeatureUnavailable();
	}
	DocumentEntriesModel->DeleteItems(DocumentEntriesModel->FindItemsByResult(orSuccess));
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanDeleteAllInSuccess()
{
	return DocumentEntriesModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::ResetResult()
{
	if(!CanResetResult()) {
		HandleFeatureUnavailable();
	}
	TIntegerDynArray LIndexes = DocumentEntriesSelection->SelectionIndexes;
	for(int LI = 0; LI < LIndexes.Length; ++LI) {
		DocumentEntriesModel->Items[LIndexes[LI]]->Result = orNone;
	}
	DocumentEntriesModel->ItemsChange(LIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntriesCommands::CanResetResult()
{
	return DocumentEntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".AddRegularFilesDialog";
	{
		AddRegularFilesDialog->FilterIndex = ASettings->ReadInteger(
			LSection,
			"FilterIndex",
			AddRegularFilesDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntriesCommands::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".AddRegularFilesDialog";
	{
		ASettings->WriteInteger(
			LSection,
			"FilterIndex",
			AddRegularFilesDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------
} // End namespace App_documents
//---------------------------------------------------------------------------

