//---------------------------------------------------------------------------
#ifndef App_DocumentsH
#define App_DocumentsH
//---------------------------------------------------------------------------
#include <FileCtrl.hpp>
#include "App_Common.h"
#include "Moa_Lists.h"
//---------------------------------------------------------------------------
namespace Comctrls {class TListView;} using namespace Comctrls;
namespace Dialogs {class TOpenDialog;} using namespace Dialogs;
namespace Moa_clipboard {class TClipboardWatcher;} using namespace Moa_clipboard;
namespace Moa_shellfiles {class TFileTypeCache;} using namespace Moa_shellfiles;
//---------------------------------------------------------------------------
namespace App_documents {
//---------------------------------------------------------------------------
class EDocumentEntryError : public EAppError
{
public:
	__fastcall EDocumentEntryError(const AnsiString& AMessage) : EAppError(AMessage) {}
};
//---------------------------------------------------------------------------
class EDocumentEntryCreateError : public EDocumentEntryError
{
public:
	__fastcall EDocumentEntryCreateError(const AnsiString& AMessage) : EDocumentEntryError(AMessage) {}
};
//---------------------------------------------------------------------------
enum TOpenResult
{
	orNone,
	orSuccess,
	orFailure,
};
//---------------------------------------------------------------------------
class TDocumentEntry : public TObject
{
private:
	AnsiString FPath;
	AnsiString FDirectory;
	AnsiString FName;
	AnsiString FType;
	DWORDLONG FSize;
	FILETIME FLastWriteTime;
	DWORD FAttributes;
	TOpenResult FResult;
	void __fastcall Initialize(const AnsiString& AFile);
	void __fastcall Uninitialize();
	__property DWORD Attributes = {read=FAttributes};
protected:
public:
	__fastcall TDocumentEntry(const AnsiString& AFile);
	virtual __fastcall ~TDocumentEntry();
	virtual bool __fastcall IsRegularFile();
	virtual bool __fastcall IsDirectory();
	__property AnsiString Path = {read=FPath};
	__property AnsiString Directory = {read=FDirectory};
	__property AnsiString Name = {read=FName};
	__property AnsiString Type = {read=FType};
	__property DWORDLONG Size = {read=FSize};
	__property FILETIME LastWriteTime = {read=FLastWriteTime};
	__property TOpenResult Result = {read=FResult, write=FResult};
};
//---------------------------------------------------------------------------
class TDocumentEntryDescriber
{
private:
protected:
public:
	static int ImageIndex_RegularFile;
	static int ImageIndex_Directory;
	static int StateIndex_None;
	static int StateIndex_Success;
	static int StateIndex_Failure;
	static AnsiString __fastcall DescribeByPath(TDocumentEntry* ADocumentEntry);
	static AnsiString __fastcall DescribeByDirectory(TDocumentEntry* ADocumentEntry);
	static AnsiString __fastcall DescribeByName(TDocumentEntry* ADocumentEntry);
	static AnsiString __fastcall DescribeBySize(TDocumentEntry* ADocumentEntry);
	static AnsiString __fastcall DescribeByType(TDocumentEntry* ADocumentEntry);
	static AnsiString __fastcall DescribeByLastWriteTime(TDocumentEntry* ADocumentEntry);
	static AnsiString __fastcall DescribeByResult(TDocumentEntry* ADocumentEntry);
	static AnsiString __fastcall DescribePath(const AnsiString& APath);
	static AnsiString __fastcall DescribeType(const AnsiString& AType);
	static AnsiString __fastcall DescribeTime(const FILETIME& ATime);
	static AnsiString __fastcall DescribeSize(DWORDLONG ASize);
	static AnsiString __fastcall DescribeResult(TOpenResult AResult);
	static int __fastcall CompareByPath(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2);
	static int __fastcall CompareByDirectory(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2);
	static int __fastcall CompareByName(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2);
	static int __fastcall CompareBySize(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2);
	static int __fastcall CompareByType(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2);
	static int __fastcall CompareByLastWriteTime(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2);
	static int __fastcall CompareByResult(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2);
	static int __fastcall ComparePath(const AnsiString& APath1, const AnsiString& APath2);
	static int __fastcall CompareType(const AnsiString& AType1, const AnsiString& AType2);
	static int __fastcall CompareTime(const FILETIME& ATime1, const FILETIME& ATime2);
	static int __fastcall CompareSize(DWORDLONG ASize1, DWORDLONG ASize2);
	static int __fastcall CompareResult(TOpenResult AResult1, TOpenResult AResult2);
	static int __fastcall EvaluateResult(TOpenResult AResult);
	static int __fastcall DetermineImageIndex(TDocumentEntry* ADocumentEntry);
	static int __fastcall DetermineStateIndex(TDocumentEntry* ADocumentEntry);
};
//---------------------------------------------------------------------------
class TDocumentEntriesModel : public TModel
{
private:
	TObjectList* FList;
	TInsertItemsEvent FOnInsertItems;
	TDeleteItemsEvent FOnDeleteItems;
	TNotifyEvent FOnClear;
	TReorderEvent FOnReorder;
	TItemsChangeEvent FOnItemsChange;
	static int __fastcall SortCompareByPath(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByDirectory(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByName(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareBySize(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByType(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByLastWriteTime(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByResult(int AIndex1, int AIndex2, void* AData);
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	TDocumentEntry* __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	int __fastcall GetCapacity();
	void __fastcall SetCapacity(int ACapacity);
	__property TObjectList* List = {read=FList};
protected:
	virtual void __fastcall DoInsertItems(const TIntegerDynArray& AIndexes);
	virtual void __fastcall DoDeleteItems(const TIntegerDynArray& AIndexes);
	virtual void __fastcall DoClear();
	virtual void __fastcall DoReorder(const TReorderRecords& ARecords);
	virtual void __fastcall DoItemsChange(const TIntegerDynArray& AIndexes);
public:
	__fastcall TDocumentEntriesModel();
	virtual __fastcall ~TDocumentEntriesModel();
	virtual void __fastcall AddItems(const DynamicArray<TDocumentEntry*>& AItems);
	virtual void __fastcall DeleteItems(const TIntegerDynArray& AIndexes);
	virtual void __fastcall Clear();
	virtual void __fastcall SortByPath();
	virtual void __fastcall SortByDirectory();
	virtual void __fastcall SortByName();
	virtual void __fastcall SortBySize();
	virtual void __fastcall SortByType();
	virtual void __fastcall SortByLastWriteTime();
	virtual void __fastcall SortByResult();
	virtual void __fastcall ReverseOrder();
	virtual void __fastcall MoveUp(const TIntegerDynArray& AIndexes);
	virtual void __fastcall MoveDown(const TIntegerDynArray& AIndexes);
	virtual void __fastcall Reorder(const TReorderRecords& ARecords);
	virtual void __fastcall ItemsChange(const TIntegerDynArray& AIndexes);
	virtual int __fastcall Find(TDocumentEntry* AItem);
	virtual int __fastcall FindByResult(TOpenResult AResult);
	virtual TIntegerDynArray __fastcall FindItemsByResult(TOpenResult AResult);
	virtual int __fastcall CountByResult(TOpenResult AResult);
	virtual int __fastcall CountByResultIn(TOpenResult AResult, const TIntegerDynArray& AIndexes);
	__property TDocumentEntry* Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount};
	__property int Capacity = {read=GetCapacity, write=SetCapacity};
	__property TInsertItemsEvent OnInsertItems = {read=FOnInsertItems, write=FOnInsertItems};
	__property TDeleteItemsEvent OnDeleteItems = {read=FOnDeleteItems, write=FOnDeleteItems};
	__property TNotifyEvent OnClear = {read=FOnClear, write=FOnClear};
	__property TReorderEvent OnReorder = {read=FOnReorder, write=FOnReorder};
	__property TItemsChangeEvent OnItemsChange = {read=FOnItemsChange, write=FOnItemsChange};
};
//---------------------------------------------------------------------------
class TDocumentEntriesSelection : public TSelection
{
private:
	TDocumentEntriesModel* FDocumentEntriesModel;
	TListView* FListView;
	bool __fastcall GetItem(int AIndex);
	void __fastcall SetItem(int AIndex, bool AItem);
	int __fastcall GetCount();
	int __fastcall GetSelectionCount();
	TIntegerDynArray __fastcall GetSelectionIndexes();
	void __fastcall SetSelectionIndexes(const TIntegerDynArray& ASelectionIndexes);
	__property TDocumentEntriesModel* DocumentEntriesModel = {read=FDocumentEntriesModel};
	__property TListView* ListView = {read=FListView};
protected:
public:
	__fastcall TDocumentEntriesSelection(TDocumentEntriesModel* ADocumentEntriesModel, TListView* AListView);
	virtual __fastcall ~TDocumentEntriesSelection();
	virtual void __fastcall HandleInsertItems(TObject* ASender, const TIntegerDynArray& AIndexes);
	virtual void __fastcall HandleDeleteItems(TObject* ASender, const TIntegerDynArray& AIndexes);
	virtual void __fastcall HandleClear(TObject* ASender);
	virtual void __fastcall HandleReorder(TObject* ASender, const TReorderRecords& ARecords);
	virtual void __fastcall HandleItemsChange(TObject* ASender, const TIntegerDynArray& AIndexes);
	virtual void __fastcall SelectAll();
	virtual void __fastcall ClearSelection();
	virtual void __fastcall ReverseSelection();
	__property bool Items[int AIndex] = {read=GetItem, write=SetItem};
	__property int Count = {read=GetCount};
	__property int SelectionCount = {read=GetSelectionCount};
	__property TIntegerDynArray SelectionIndexes = {read=GetSelectionIndexes, write=SetSelectionIndexes};
};
//---------------------------------------------------------------------------
class TDocumentEntriesCommands : public TCommands
{
private:
	TDocumentEntriesModel* FDocumentEntriesModel;
	TDocumentEntriesSelection* FDocumentEntriesSelection;
	TClipboardWatcher* FClipboardWatcher;
	TOpenDialog* FAddRegularFilesDialog;
	TNotifyEvent FOnChange;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall ClipboardWatcherChange(TObject* ASender);
	__property TDocumentEntriesModel* DocumentEntriesModel = {read=FDocumentEntriesModel};
	__property TDocumentEntriesSelection* DocumentEntriesSelection = {read=FDocumentEntriesSelection};
	__property TClipboardWatcher* ClipboardWatcher = {read=FClipboardWatcher};
	__property TOpenDialog* AddRegularFilesDialog = {read=FAddRegularFilesDialog};
protected:
	virtual void __fastcall DoChange();
public:
	__fastcall TDocumentEntriesCommands(TDocumentEntriesModel* ADocumentEntriesModel, TDocumentEntriesSelection* ADocumentEntriesSelection);
	virtual __fastcall ~TDocumentEntriesCommands();
	virtual void __fastcall New();
	virtual bool __fastcall CanNew();
	virtual void __fastcall AddRegularFiles();
	virtual bool __fastcall CanAddRegularFiles();
	virtual void __fastcall AddDirectory();
	virtual bool __fastcall CanAddDirectory();
	virtual void __fastcall AddDroppedFiles(const TStringDynArray& ADroppedFiles);
	virtual bool __fastcall CanAddDroppedFiles();
	virtual void __fastcall AddFiles(const TStringDynArray& AFiles);
	virtual bool __fastcall CanAddFiles();
	virtual void __fastcall Cut();
	virtual bool __fastcall CanCut();
	virtual void __fastcall Copy();
	virtual bool __fastcall CanCopy();
	virtual void __fastcall Paste();
	virtual bool __fastcall CanPaste();
	virtual void __fastcall Delete();
	virtual void __fastcall Delete(bool AConfirm);
	virtual bool __fastcall CanDelete();
	virtual void __fastcall SelectAll();
	virtual bool __fastcall CanSelectAll();
	virtual void __fastcall ReverseSelection();
	virtual bool __fastcall CanReverseSelection();
	virtual void __fastcall SortByPath();
	virtual bool __fastcall CanSortByPath();
	virtual void __fastcall SortByDirectory();
	virtual bool __fastcall CanSortByDirectory();
	virtual void __fastcall SortByName();
	virtual bool __fastcall CanSortByName();
	virtual void __fastcall SortBySize();
	virtual bool __fastcall CanSortBySize();
	virtual void __fastcall SortByType();
	virtual bool __fastcall CanSortByType();
	virtual void __fastcall SortByLastWriteTime();
	virtual bool __fastcall CanSortByLastWriteTime();
	virtual void __fastcall SortByResult();
	virtual bool __fastcall CanSortByResult();
	virtual void __fastcall ReverseOrder();
	virtual bool __fastcall CanReverseOrder();
	virtual void __fastcall MoveUp();
	virtual bool __fastcall CanMoveUp();
	virtual void __fastcall MoveDown();
	virtual bool __fastcall CanMoveDown();
	virtual void __fastcall DeleteAllInSuccess();
	virtual bool __fastcall CanDeleteAllInSuccess();
	virtual void __fastcall ResetResult();
	virtual bool __fastcall CanResetResult();
	virtual void __fastcall LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual void __fastcall SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	__property TNotifyEvent OnChange = {read=FOnChange, write=FOnChange};
};
//---------------------------------------------------------------------------
} // End namespace App_documents
using namespace App_documents;
//---------------------------------------------------------------------------
#endif

