//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_Open.h"
#include "App_Processes.h"
#include "App_ProgressDialog.h"
#include "Moa_AutoPointers.h"
#include "Moa_DynamicArrays.h"
#include "Moa_Tasks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_open {
//---------------------------------------------------------------------------
typedef void __fastcall (__closure * TOpenNextEvent)(TObject* ASender, const AnsiString& AProgram, const AnsiString& ADocument);
typedef void __fastcall (__closure * TOpenMessageEvent)(TObject* ASender, const AnsiString& AMessage);
//---------------------------------------------------------------------------
class TOpenTask : public TTask
{
private:
	class EOpenSkip : public EAbort
	{
	public:
		__fastcall EOpenSkip(const AnsiString& AMessage) : EAbort(AMessage) {}
	};
	struct TPrepareArguments
	{
		bool Skip;
		__fastcall TPrepareArguments(bool ASkip) : Skip(ASkip) {}
	};
	struct TMarkArguments
	{
		TOpenResult Result;
		__fastcall TMarkArguments(TOpenResult AResult) : Result(AResult) {}
	};
	struct TOnNextArguments
	{
		const AnsiString& Program;
		const AnsiString& Document;
		__fastcall TOnNextArguments(const AnsiString& AProgram, const AnsiString& ADocument) : Program(AProgram), Document(ADocument) {}
	};
	struct TOnInformationArguments
	{
		const AnsiString& Message;
		__fastcall TOnInformationArguments(const AnsiString& AMessage) : Message(AMessage) {}
	};
	typedef TOnInformationArguments TOnWarningArguments;
	typedef TOnInformationArguments TOnErrorArguments;
	TProgramEntry* FProgramEntry;
	TDocumentEntriesModel* FDocumentEntriesModel;
	TProcessCreator* FProcessCreator;
	int FOpenInterval;
	int FCurrent;
	int FAll;
	bool FPaused;
	TOpenNextEvent FOnNext;
	TNotifyEvent FOnPause;
	TNotifyEvent FOnResume;
	TOpenMessageEvent FOnInformation;
	TOpenMessageEvent FOnWarning;
	TOpenMessageEvent FOnError;
	__property TProgramEntry* ProgramEntry = {read=FProgramEntry};
	__property TDocumentEntriesModel* DocumentEntriesModel = {read=FDocumentEntriesModel};
	__property TProcessCreator* ProcessCreator = {read=FProcessCreator};
	__property int OpenInterval = {read=FOpenInterval};
	__property int Current = {read=FCurrent};
	__property int All = {read=FAll};
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall ResetResults();
	void __fastcall Process();
	void __fastcall Prepare();
	void __fastcall Unprepare();
	void __fastcall Mark(TOpenResult AResult);
	void __fastcall Continue();
	void __fastcall SynchronizedResetResults(void* AData);
	void __fastcall SynchronizedPrepare(void* AData);
	void __fastcall SynchronizedUnprepare(void* AData);
	void __fastcall SynchronizedMark(void* AData);
	void __fastcall SynchronizedDoNext(void* AData);
	void __fastcall SynchronizedDoPause(void* AData);
	void __fastcall SynchronizedDoResume(void* AData);
	void __fastcall SynchronizedDoInformation(void* AData);
	void __fastcall SynchronizedDoWarning(void* AData);
	void __fastcall SynchronizedDoError(void* AData);
protected:
	virtual void __fastcall Execute();
	virtual void __fastcall DoNext(const AnsiString& AProgram, const AnsiString& ADocument);
	virtual void __fastcall DoPause();
	virtual void __fastcall DoResume();
	virtual void __fastcall DoInformation(const AnsiString& AMessage);
	virtual void __fastcall DoWarning(const AnsiString& AMessage);
	virtual void __fastcall DoError(const AnsiString& AMessage);
public:
	__fastcall TOpenTask(TProgramEntry* AProgramEntry, TDocumentEntriesModel* ADocumentEntriesModel);
	virtual __fastcall ~TOpenTask();
	virtual void __fastcall Pause();
	virtual void __fastcall Resume();
	virtual void __fastcall HandlePause(TObject* ASender);
	virtual void __fastcall HandleResume(TObject* ASender);
	virtual void __fastcall HandleCancel(TObject* ASender);
	__property bool Paused = {read=FPaused};
	__property TOpenNextEvent OnNext = {read=FOnNext, write=FOnNext};
	__property TNotifyEvent OnPause = {read=FOnPause, write=FOnPause};
	__property TNotifyEvent OnResume = {read=FOnResume, write=FOnResume};
	__property TOpenMessageEvent OnInformation = {read=FOnInformation, write=FOnInformation};
	__property TOpenMessageEvent OnWarning = {read=FOnWarning, write=FOnWarning};
	__property TOpenMessageEvent OnError = {read=FOnError, write=FOnError};
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TOpenTask::Initialize()
{
	try {
		FProcessCreator = new TProcessCreator();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::Uninitialize()
{
	delete FProcessCreator;
	FProcessCreator = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::ResetResults()
{
	Synchronize(&SynchronizedResetResults, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::Process()
{
	try {
		Prepare();

		// Open a document
		DoInformation("Opening a document...");
		TAutoPointer<TProcess> LProcess = ProcessCreator->Execute();
		Mark(orSuccess);

		// Wait for the termination of the process
		DoInformation("Waiting for the termination of the process...");
		while(!Canceled && !LProcess->IsTerminated(100)) {}

		::Sleep(OpenInterval);
	}
	catch(EOpenSkip& LException) {
		// Do nothing
	}
	catch(EProcessCreateError& LException) {
		DoError("Could not open the document");
		Mark(orFailure);
		Pause();
		Continue();
	}
	catch(EProcessAbandon& LException) {
		DoWarning("Paused - resume to open the next document");
		Mark(orSuccess);
		Pause();
		Continue();
	}
	catch(EProcessUnavailable& LException) {
		DoWarning("Cannot wait for the termination of the process");
		Mark(orSuccess);
		Pause();
		Continue();
	}
	catch(EProcessWaitFail& LException) {
		DoError("Could not wait for termination of the process");
		Pause();
		Continue();
	}
	Unprepare();
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::Prepare()
{
	TPrepareArguments LArguments(false);
	Synchronize(&SynchronizedPrepare, &LArguments);
	if(LArguments.Skip) {
		throw EOpenSkip("The program skips the document.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::Unprepare()
{
	Synchronize(&SynchronizedUnprepare, NULL);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::Mark(TOpenResult AResult)
{
	TMarkArguments LArguments(AResult);
	Synchronize(&SynchronizedMark, &LArguments);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::Continue()
{
	while(Paused && !Canceled) {
		::Sleep(100);
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedResetResults(void* AData)
{
	DoInformation("Resetting the results...");
	TAutoPointer<TDynamicArrayBuilder<int> > LBuilder = new TDynamicArrayBuilder<int>();
	LBuilder->Capacity = DocumentEntriesModel->Count - DocumentEntriesModel->CountByResult(orSuccess);
	for(int LI = 0; LI < DocumentEntriesModel->Count; ++LI) {
		if(DocumentEntriesModel->Items[LI]->Result != orSuccess) {
			DocumentEntriesModel->Items[LI]->Result = orNone;
			LBuilder->Add(LI);
		}
	}
	DocumentEntriesModel->ItemsChange(LBuilder->Finish());
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedPrepare(void* AData)
{
	TDocumentEntry* LDocumentEntry = DocumentEntriesModel->Items[Current];
	DoNext(ProgramEntry->DisplayName, LDocumentEntry->Name);
	{
		TPrepareArguments& LArguments = *static_cast<TPrepareArguments*>(AData);
		LArguments.Skip = LDocumentEntry->Result == orSuccess;
		if(LArguments.Skip) {
			return;
		}
	}
	if(ProgramEntry->IsDefault()) {
		ProcessCreator->File = LDocumentEntry->Path;
		ProcessCreator->Parameters = AnsiString();
	}
	else {
		ProcessCreator->File = ProgramEntry->File;
		ProcessCreator->Parameters = ProgramEntry->DocumentPrefix + LDocumentEntry->Path + ProgramEntry->DocumentSuffix;
	}
	if(ProgramEntry->WorkingDirectory.Length() == 0) {
		ProcessCreator->WorkingDirectory = ExtractFileDir(LDocumentEntry->Path);
	}
	else {
		ProcessCreator->WorkingDirectory = ProgramEntry->WorkingDirectory;
	}
	ProcessCreator->ShowCommand = ProgramEntry->ShowCommand;
	ProcessCreator->AbandonProcess = ProgramEntry->AutoPause;
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedUnprepare(void* AData)
{
	DoInformation("Done");
	++FCurrent;
	DoProgress(Current, All);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedMark(void* AData)
{
	TMarkArguments& LArguments = *static_cast<TMarkArguments*>(AData);
	DocumentEntriesModel->Items[Current]->Result = LArguments.Result;
	DocumentEntriesModel->ItemsChange(TDynamicArrayUtilities<int>::FromElements(Current));
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedDoNext(void* AData)
{
	TOnNextArguments& LArguments = *static_cast<TOnNextArguments*>(AData);
	OnNext(this, LArguments.Program, LArguments.Document);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedDoPause(void* AData)
{
	OnPause(this);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedDoResume(void* AData)
{
	OnResume(this);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedDoInformation(void* AData)
{
	TOnInformationArguments& LArguments = *static_cast<TOnInformationArguments*>(AData);
	OnInformation(this, LArguments.Message);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedDoWarning(void* AData)
{
	TOnWarningArguments& LArguments = *static_cast<TOnWarningArguments*>(AData);
	OnWarning(this, LArguments.Message);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedDoError(void* AData)
{
	TOnErrorArguments& LArguments = *static_cast<TOnErrorArguments*>(AData);
	OnError(this, LArguments.Message);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::Execute()
{
	ResetResults();
	while(!Canceled && Current < All) {
		Process();
		Continue();
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::DoNext(const AnsiString& AProgram, const AnsiString& ADocument)
{
	if(OnNext != NULL) {
		TOnNextArguments LArguments(AProgram, ADocument);
		Synchronize(&SynchronizedDoNext, &LArguments);
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::DoPause()
{
	if(OnPause != NULL) {
		Synchronize(&SynchronizedDoPause, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::DoResume()
{
	if(OnResume != NULL) {
		Synchronize(&SynchronizedDoResume, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::DoInformation(const AnsiString& AMessage)
{
	if(OnInformation != NULL) {
		TOnInformationArguments LArguments(AMessage);
		Synchronize(&SynchronizedDoInformation, &LArguments);
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::DoWarning(const AnsiString& AMessage)
{
	if(OnWarning != NULL) {
		TOnInformationArguments LArguments(AMessage);
		Synchronize(&SynchronizedDoWarning, &LArguments);
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::DoError(const AnsiString& AMessage)
{
	if(OnError != NULL) {
		TOnInformationArguments LArguments(AMessage);
		Synchronize(&SynchronizedDoError, &LArguments);
	}
}
//---------------------------------------------------------------------------
__fastcall TOpenTask::TOpenTask(TProgramEntry* AProgramEntry, TDocumentEntriesModel* ADocumentEntriesModel) :
	TTask(),
	FProgramEntry(AProgramEntry),
	FDocumentEntriesModel(ADocumentEntriesModel),
	FProcessCreator(NULL),
	FOpenInterval(AProgramEntry->OpenInterval),
	FCurrent(0),
	FAll(ADocumentEntriesModel->Count),
	FPaused(false),
	FOnNext(NULL),
	FOnPause(NULL),
	FOnResume(NULL),
	FOnInformation(NULL),
	FOnWarning(NULL),
	FOnError(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TOpenTask::~TOpenTask()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::Pause()
{
	if(Paused) {
		return;
	}
	FPaused = true;
	DoPause();
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::Resume()
{
	if(!Paused) {
		return;
	}
	FPaused = false;
	DoResume();
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::HandlePause(TObject* ASender)
{
	Pause();
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::HandleResume(TObject* ASender)
{
	Resume();
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::HandleCancel(TObject* ASender)
{
	Cancel();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TOpenCommands::TOpenCommands(
	TProgramEntriesModel* AProgramEntriesModel,
	TProgramEntriesSelection* AProgramEntriesSelection,
	TDocumentEntriesModel* ADocumentEntriesModel) :
	TCommands(),
	FProgramEntriesModel(AProgramEntriesModel),
	FProgramEntriesSelection(AProgramEntriesSelection),
	FDocumentEntriesModel(ADocumentEntriesModel)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TOpenCommands::~TOpenCommands()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TOpenCommands::Open()
{
	if(!CanOpen()) {
		HandleFeatureUnavailable();
	}
	TAutoPointer<TOpenTask> LTask = new TOpenTask(
		ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex],
		DocumentEntriesModel);
	TAppProgressDialog::EnsureCreated();
	AppProgressDialog->OnPause = LTask->HandlePause;
	AppProgressDialog->OnResume = LTask->HandleResume;
	AppProgressDialog->OnCancel = LTask->HandleCancel;
	LTask->FreeOnTerminate = true;
	LTask->OnTerminate = AppProgressDialog->HandleTerminate;
	LTask->OnNext = AppProgressDialog->HandleNext;
	LTask->OnPause = AppProgressDialog->HandlePause;
	LTask->OnResume = AppProgressDialog->HandleResume;
	LTask->OnInformation = AppProgressDialog->HandleInformation;
	LTask->OnWarning = AppProgressDialog->HandleWarning;
	LTask->OnError = AppProgressDialog->HandleError;
	LTask->OnProgress = AppProgressDialog->HandleProgress;
	LTask->OnException = Application->OnException;
	LTask->Perform();
	LTask.Release();
	AppProgressDialog->Execute();
}
//---------------------------------------------------------------------------
bool __fastcall TOpenCommands::CanOpen()
{
	return
		ProgramEntriesSelection->SelectionIndex >= 0 &&
		DocumentEntriesModel->Count > 0;
}
//---------------------------------------------------------------------------
} // End namespace App_open
//---------------------------------------------------------------------------

