//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "App_ProgramDialog.h"
#include "App_Programs.h"
#include "Moa_AutoPointers.h"
#include "Moa_Files.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_programs {
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Initialize()
{
	try {
		FIcon = new TIcon();
		UpdateIcon();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Uninitialize()
{
	delete FIcon;
	FIcon = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::UpdateIcon()
{
	HICON LHandle = NULL;
	{
		LPCTSTR LPath;
		DWORD LFileAttributes;
		UINT LFlags;
		if(IsDefault()) {
			LPath = "Program.exe";
			LFileAttributes = FILE_ATTRIBUTE_NORMAL;
			LFlags = SHGFI_ICON | SHGFI_SHELLICONSIZE | SHGFI_SMALLICON | SHGFI_USEFILEATTRIBUTES;
		}
		else {
			LPath = FFile.c_str();
			LFileAttributes = 0;
			LFlags = SHGFI_ICON | SHGFI_SHELLICONSIZE | SHGFI_SMALLICON;
		}
		SHFILEINFO LFileInfo = {0};
		if(::SHGetFileInfo(
			LPath,
			LFileAttributes,
			&LFileInfo,
			sizeof(SHFILEINFO),
			LFlags) != NULL) {
			LHandle = ::CopyIcon(LFileInfo.hIcon);
		}
	}
	try {
		Icon->Handle = LHandle;
	}
	catch(...) {
		if(LHandle != NULL) {
			::DestroyIcon(LHandle);
		}
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::SetFile(const AnsiString& AFile)
{
	AnsiString LFile = AFile;
	if(LFile.Length() > 0) {
		try {
			TPathUtilities::AssureAbsolute(LFile);
			LFile = TFileUtilities::GetLongPath(ExpandFileName(LFile));
		}
		catch(EFileError& LException) {
			throw EProgramEntryError(AnsiString().sprintf(
				"The program cannot accept the path \"%s\".",
				AFile.c_str()));
		}
	}
	if(LFile == File) {
		return;
	}
	FFile = LFile;
	UpdateIcon();
}
//---------------------------------------------------------------------------
__fastcall TProgramEntry::TProgramEntry() :
	TObject(),
	FIcon(NULL),
	FDisplayName(),
	FFile(),
	FDocumentPrefix(),
	FDocumentSuffix(),
	FWorkingDirectory(),
	FShowCommand(SW_SHOWNORMAL),
	FOpenInterval(0),
	FAutoPause(true)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TProgramEntry::~TProgramEntry()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Assign(TProgramEntry* ASource)
{
	if(ASource == this) {
		return;
	}
	FIcon->Assign(ASource->Icon);
	FDisplayName = ASource->FDisplayName;
	FFile = ASource->FFile;
	FDocumentPrefix = ASource->FDocumentPrefix;
	FDocumentSuffix = ASource->FDocumentSuffix;
	FWorkingDirectory = ASource->FWorkingDirectory;
	FShowCommand = ASource->FShowCommand;
	FOpenInterval = ASource->FOpenInterval;
	FAutoPause = ASource->FAutoPause;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Load(TMemIniFile* ASettings, const AnsiString& ASection)
{
	try {
		TAutoPointer<TProgramEntry> LInstance = new TProgramEntry();
		LInstance->DisplayName = TAnsiStringUtilities::Dequote(ASettings->ReadString(
			ASection,
			"DisplayName",
			TAnsiStringUtilities::Quote(LInstance->DisplayName)));
		LInstance->File = TAnsiStringUtilities::Dequote(ASettings->ReadString(
			ASection,
			"File",
			TAnsiStringUtilities::Quote(LInstance->File)));
		LInstance->DocumentPrefix = TAnsiStringUtilities::Dequote(ASettings->ReadString(
			ASection,
			"DocumentPrefix",
			TAnsiStringUtilities::Quote(LInstance->DocumentPrefix)));
		LInstance->DocumentSuffix = TAnsiStringUtilities::Dequote(ASettings->ReadString(
			ASection,
			"DocumentSuffix",
			TAnsiStringUtilities::Quote(LInstance->DocumentSuffix)));
		LInstance->WorkingDirectory = TAnsiStringUtilities::Dequote(ASettings->ReadString(
			ASection,
			"WorkingDirectory",
			TAnsiStringUtilities::Quote(LInstance->WorkingDirectory)));
		LInstance->ShowCommand = ASettings->ReadInteger(
			ASection,
			"ShowCommand",
			LInstance->ShowCommand);
		LInstance->OpenInterval = ASettings->ReadInteger(
			ASection,
			"OpenInterval",
			LInstance->OpenInterval);
		LInstance->AutoPause = ASettings->ReadBool(
			ASection,
			"AutoPause",
			LInstance->AutoPause);
		Assign(LInstance);
	}
	catch(EProgramEntryError& LException) {
		throw EProgramEntryLoadError(LException.Message);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Save(TMemIniFile* ASettings, const AnsiString& ASection)
{
	try {
		ASettings->WriteString(
			ASection,
			"DisplayName",
			TAnsiStringUtilities::Quote(DisplayName));
		ASettings->WriteString(
			ASection,
			"File",
			TAnsiStringUtilities::Quote(File));
		ASettings->WriteString(
			ASection,
			"DocumentPrefix",
			TAnsiStringUtilities::Quote(DocumentPrefix));
		ASettings->WriteString(
			ASection,
			"DocumentSuffix",
			TAnsiStringUtilities::Quote(DocumentSuffix));
		ASettings->WriteString(
			ASection,
			"WorkingDirectory",
			TAnsiStringUtilities::Quote(WorkingDirectory));
		ASettings->WriteInteger(
			ASection,
			"ShowCommand",
			ShowCommand);
		ASettings->WriteInteger(
			ASection,
			"OpenInterval",
			OpenInterval);
		ASettings->WriteBool(
			ASection,
			"AutoPause",
			AutoPause);
	}
	catch(EProgramEntryError& LException) {
		throw EProgramEntrySaveError(LException.Message);
	}
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntry::IsDefault()
{
	return File.Length() == 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::Initialize()
{
	try {
		FList = new TStringList();
		FList->Duplicates = dupError;
		FList->CaseSensitive = true;
		FList->Sorted = true;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::Uninitialize()
{
	if(FList != NULL) {
		for(int LI = 0; LI < FList->Count; ++LI) {
			delete FList->Objects[LI];
		}
		delete FList;
		FList = NULL;
	}
}
//---------------------------------------------------------------------------
TProgramEntry* __fastcall TProgramEntriesModel::GetItem(int AIndex)
{
	return static_cast<TProgramEntry*>(List->Objects[AIndex]);
}
//---------------------------------------------------------------------------
int __fastcall TProgramEntriesModel::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
int __fastcall TProgramEntriesModel::GetCapacity()
{
	return List->Capacity;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::SetCapacity(int ACapacity)
{
	List->Capacity = ACapacity;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::DoInsert(int AIndex)
{
	if(OnInsert != NULL) {
		OnInsert(this, AIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::DoDelete(int AIndex)
{
	if(OnDelete != NULL) {
		OnDelete(this, AIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::DoClear()
{
	if(OnClear != NULL) {
		OnClear(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesModel::TProgramEntriesModel() :
	TModel(),
	FList(NULL),
	FOnInsert(NULL),
	FOnDelete(NULL),
	FOnClear(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesModel::~TProgramEntriesModel()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::Add(TProgramEntry* AItem)
{
	try {
		DoInsert(List->AddObject(AItem->DisplayName, AItem));
	}
	catch(EStringListError& LException) {
		throw EProgramEntriesError("The program cannot add the program entry because its display name collides with another.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::Delete(int AIndex)
{
	delete List->Objects[AIndex];
	List->Delete(AIndex);
	DoDelete(AIndex);
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::Clear()
{
	if(Count == 0) {
		return;
	}
	for(int LI = 0; LI < List->Count; ++LI) {
		delete List->Objects[LI];
	}
	List->Clear();
	DoClear();
}
//---------------------------------------------------------------------------
int __fastcall TProgramEntriesModel::Find(TProgramEntry* AItem)
{
	return List->IndexOfObject(AItem);
}
//---------------------------------------------------------------------------
int __fastcall TProgramEntriesModel::FindByDisplayName(const AnsiString& ADisplayName)
{
	int LIndex;
	if(List->Find(ADisplayName, LIndex)) {
		return LIndex;
	}
	else {
		return -1;
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TProgramEntriesModel::EnsureUniqueDisplayName(const AnsiString& ADisplayName)
{
	const int LMinIndex = 1;
	const int LMaxIndex = 100;
	AnsiString LDisplayName;
	for(int LIndex = LMinIndex; LIndex <= LMaxIndex; ++LIndex) {
		if(LIndex == LMinIndex) {
			LDisplayName = ADisplayName;
		}
		else {
			LDisplayName = AnsiString().sprintf("%s (%d)", ADisplayName.c_str(), LIndex);
		}
		if(FindByDisplayName(LDisplayName) >= 0) {
			if(LIndex == LMaxIndex) {
				throw EAppError("The program could not provide a unique display name.");
			}
			continue;
		}
		break;
	}
	return LDisplayName;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection;
	if(ASettings->SectionExists(LSection)) {
		Clear();
		int LCount = ASettings->ReadInteger(LSection, "Count", 0);
		Capacity = LCount;
		for(int LI = 0; LI < LCount; ++LI) {
			try {
				TAutoPointer<TProgramEntry> LItem = new TProgramEntry();
				LItem->Load(ASettings, LSection + AnsiString().sprintf(".Items[%d]", LI));
				Add(LItem);
				LItem.Release();
			}
			catch(EProgramEntryLoadError& LException) {
				// Do nothing
			}
		}
		if(Count != LCount) {
			Application->MessageBox(
				"The program could not load all the program entries.",
				Application->Title.c_str(),
				MB_OK | MB_ICONERROR | MB_SETFOREGROUND);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesModel::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection;
	{
		ASettings->WriteInteger(LSection, "Count", Count);
		for(int LI = 0; LI < Count; ++LI) {
			Items[LI]->Save(ASettings, LSection + AnsiString().sprintf(".Items[%d]", LI));
		}
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int __fastcall TProgramEntriesSelection::GetCount()
{
	return ComboBox->Items->Count;
}
//---------------------------------------------------------------------------
int __fastcall TProgramEntriesSelection::GetSelectionIndex()
{
	return ComboBox->ItemIndex;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::SetSelectionIndex(int ASelectionIndex)
{
	ComboBox->ItemIndex = ASelectionIndex;
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesSelection::TProgramEntriesSelection(TProgramEntriesModel* AProgramEntriesModel, TComboBox* AComboBox) :
	TSettingsHandler(),
	FProgramEntriesModel(AProgramEntriesModel),
	FComboBox(AComboBox)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesSelection::~TProgramEntriesSelection()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::HandleInsert(TObject* ASender, int AIndex)
{
	ComboBox->Items->BeginUpdate();
	try {
		ComboBox->Items->InsertObject(
			AIndex,
			ProgramEntriesModel->Items[AIndex]->DisplayName,
			ProgramEntriesModel->Items[AIndex]);
		ComboBox->ItemIndex = AIndex;
	}
	catch(...) {
		ComboBox->Items->EndUpdate();
		throw;
	}
	ComboBox->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::HandleDelete(TObject* ASender, int AIndex)
{
	ComboBox->Items->BeginUpdate();
	try {
		int LItemIndex = ComboBox->ItemIndex;
		if(AIndex < LItemIndex || LItemIndex == ProgramEntriesModel->Count) {
			--LItemIndex;
		}
		ComboBox->Items->Delete(AIndex);
		ComboBox->ItemIndex = LItemIndex;
	}
	catch(...) {
		ComboBox->Items->EndUpdate();
		throw;
	}
	ComboBox->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::HandleClear(TObject* ASender)
{
	if(ComboBox->Items->Count == 0) {
		return;
	}
	ComboBox->Items->BeginUpdate();
	try {
		ComboBox->Items->Clear();
	}
	catch(...) {
		ComboBox->Items->EndUpdate();
		throw;
	}
	ComboBox->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection;
	{
		int LSelectionIndex = ASettings->ReadInteger(
			LSection,
			"SelectionIndex",
			LSelectionIndex);
		if(LSelectionIndex >= Count) {
			LSelectionIndex = Count - 1;
		}
		SelectionIndex = LSelectionIndex;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesSelection::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection;
	{
		ASettings->WriteInteger(
			LSection,
			"SelectionIndex",
			SelectionIndex);
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Initialize()
{
	try {
		TAutoPointer<TProgramEntry> LItem = new TProgramEntry();
		LItem->DisplayName = "Default - depends on file association";
		ProgramEntriesModel->Add(LItem);
		LItem.Release();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::ProgramDialogDisplayNameInputForAdd(TObject* ASender, const AnsiString& ADisplayName, bool& AAccept)
{
	AAccept = ProgramEntriesModel->FindByDisplayName(ADisplayName) < 0;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::ProgramDialogDisplayNameInputForDuplicate(TObject* ASender, const AnsiString& ADisplayName, bool& AAccept)
{
	AAccept = ProgramEntriesModel->FindByDisplayName(ADisplayName) < 0;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::ProgramDialogDisplayNameInputForProperties(TObject* ASender, const AnsiString& ADisplayName, bool& AAccept)
{
	int LIndex = ProgramEntriesModel->FindByDisplayName(ADisplayName);
	AAccept = LIndex < 0 || LIndex == ProgramEntriesSelection->SelectionIndex;
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesCommands::TProgramEntriesCommands(TProgramEntriesModel* AProgramEntriesModel, TProgramEntriesSelection* AProgramEntriesSelection) :
	TCommands(),
	FProgramEntriesModel(AProgramEntriesModel),
	FProgramEntriesSelection(AProgramEntriesSelection)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TProgramEntriesCommands::~TProgramEntriesCommands()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Add()
{
	if(!CanAdd()) {
		HandleFeatureUnavailable();
	}
	TAppProgramDialog::EnsureCreated();
	AppProgramDialog->Caption = "Add";
	AppProgramDialog->Reset();
	AppProgramDialog->Values->DisplayName = ProgramEntriesModel->EnsureUniqueDisplayName(AppProgramDialog->Values->DisplayName);
	AppProgramDialog->OnDisplayNameInput = ProgramDialogDisplayNameInputForAdd;
	if(!AppProgramDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TProgramEntry> LItem = new TProgramEntry();
	LItem->Assign(AppProgramDialog->Values);
	ProgramEntriesModel->Add(LItem);
	LItem.Release();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanAdd()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Duplicate()
{
	if(!CanDuplicate()) {
		HandleFeatureUnavailable();
	}
	TAppProgramDialog::EnsureCreated();
	AppProgramDialog->Caption = "Duplicate";
	AppProgramDialog->Values->Assign(ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]);
	AppProgramDialog->Values->DisplayName = ProgramEntriesModel->EnsureUniqueDisplayName(AppProgramDialog->Values->DisplayName);
	AppProgramDialog->OnDisplayNameInput = ProgramDialogDisplayNameInputForDuplicate;
	if(!AppProgramDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TProgramEntry> LItem = new TProgramEntry();
	LItem->Assign(AppProgramDialog->Values);
	ProgramEntriesModel->Add(LItem);
	LItem.Release();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanDuplicate()
{
	return ProgramEntriesSelection->SelectionIndex >= 0;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Delete()
{
	if(!CanDelete()) {
		HandleFeatureUnavailable();
	}
	if(Application->MessageBox(
		"Do you want to remove the program from the list?",
		Application->Title.c_str(),
		MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
		Abort();
	}
	ProgramEntriesModel->Delete(ProgramEntriesSelection->SelectionIndex);
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanDelete()
{
	return ProgramEntriesSelection->SelectionIndex >= 0;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Start()
{
	if(!CanStart()) {
		HandleFeatureUnavailable();
	}
	AnsiString LFile = ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->File;
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.hwnd = Application->Handle;
	LExecuteInfo.lpFile = LFile.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw EAppError("The program could not start the program.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanStart()
{
	return
		ProgramEntriesSelection->SelectionIndex >= 0 &&
		!ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->IsDefault();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::Explore()
{
	if(!CanExplore()) {
		HandleFeatureUnavailable();
	}
	AnsiString LDirectory = ExtractFileDir(ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->File);
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.hwnd = Application->Handle;
	LExecuteInfo.lpVerb = "explore";
	LExecuteInfo.lpFile = LDirectory.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw EAppError("The program could not explore the directory.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanExplore()
{
	return
		ProgramEntriesSelection->SelectionIndex >= 0 &&
		!ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]->IsDefault();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntriesCommands::ShowProperties()
{
	if(!CanShowProperties()) {
		HandleFeatureUnavailable();
	}
	TAppProgramDialog::EnsureCreated();
	AppProgramDialog->Caption = "Properties";
	AppProgramDialog->Values->Assign(ProgramEntriesModel->Items[ProgramEntriesSelection->SelectionIndex]);
	AppProgramDialog->OnDisplayNameInput = ProgramDialogDisplayNameInputForProperties;
	if(!AppProgramDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TProgramEntry> LItem = new TProgramEntry();
	LItem->Assign(AppProgramDialog->Values);
	ProgramEntriesModel->Delete(ProgramEntriesSelection->SelectionIndex);
	ProgramEntriesModel->Add(LItem);
	LItem.Release();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramEntriesCommands::CanShowProperties()
{
	return ProgramEntriesSelection->SelectionIndex >= 0;
}
//---------------------------------------------------------------------------
} // End namespace App_programs
//---------------------------------------------------------------------------

