//---------------------------------------------------------------------------
#ifndef App_ProgramsH
#define App_ProgramsH
//---------------------------------------------------------------------------
#include "App_Common.h"
#include "Moa_Lists.h"
namespace Stdctrls {class TComboBox;} using namespace Stdctrls;
//---------------------------------------------------------------------------
namespace App_programs {
//---------------------------------------------------------------------------
class EProgramEntryError : public EAppError
{
public:
	__fastcall EProgramEntryError(const AnsiString& AMessage) : EAppError(AMessage) {}
};
//---------------------------------------------------------------------------
class EProgramEntryCreateError : public EProgramEntryError
{
public:
	__fastcall EProgramEntryCreateError(const AnsiString& AMessage) : EProgramEntryError(AMessage) {}
};
//---------------------------------------------------------------------------
class EProgramEntryLoadError : public EProgramEntryError
{
public:
	__fastcall EProgramEntryLoadError(const AnsiString& AMessage) : EProgramEntryError(AMessage) {}
};
//---------------------------------------------------------------------------
class EProgramEntrySaveError : public EProgramEntryError
{
public:
	__fastcall EProgramEntrySaveError(const AnsiString& AMessage) : EProgramEntryError(AMessage) {}
};
//---------------------------------------------------------------------------
class EProgramEntryDisplayNameCollide : public EAppError
{
public:
	__fastcall EProgramEntryDisplayNameCollide(const AnsiString& AMessage) : EAppError(AMessage) {}
};
//---------------------------------------------------------------------------
class TProgramEntry : public TObject
{
private:
	TIcon* FIcon;
	AnsiString FDisplayName;
	AnsiString FFile;
	AnsiString FDocumentPrefix;
	AnsiString FDocumentSuffix;
	AnsiString FWorkingDirectory;
	int FShowCommand;
	int FOpenInterval;
	bool FAutoPause;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall UpdateIcon();
	void __fastcall SetFile(const AnsiString& AFile);
protected:
public:
	__fastcall TProgramEntry();
	virtual __fastcall ~TProgramEntry();
	virtual void __fastcall Assign(TProgramEntry* ASource);
	virtual void __fastcall Load(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual void __fastcall Save(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual bool __fastcall IsDefault();
	__property TIcon* Icon = {read=FIcon};
	__property AnsiString DisplayName = {read=FDisplayName, write=FDisplayName};
	__property AnsiString File = {read=FFile, write=SetFile};
	__property AnsiString DocumentPrefix = {read=FDocumentPrefix, write=FDocumentPrefix};
	__property AnsiString DocumentSuffix = {read=FDocumentSuffix, write=FDocumentSuffix};
	__property AnsiString WorkingDirectory = {read=FWorkingDirectory, write=FWorkingDirectory};
	__property int ShowCommand = {read=FShowCommand, write=FShowCommand};
	__property int OpenInterval = {read=FOpenInterval, write=FOpenInterval};
	__property bool AutoPause = {read=FAutoPause, write=FAutoPause};
};
//---------------------------------------------------------------------------
class EProgramEntriesError : public EAppError
{
public:
	__fastcall EProgramEntriesError(const AnsiString& AMessage) : EAppError(AMessage) {}
};
//---------------------------------------------------------------------------
class TProgramEntriesModel : public TModel
{
private:
	TStringList* FList;
	TInsertEvent FOnInsert;
	TDeleteEvent FOnDelete;
	TNotifyEvent FOnClear;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	TProgramEntry* __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	int __fastcall GetCapacity();
	void __fastcall SetCapacity(int ACapacity);
	__property TStringList* List = {read=FList};
protected:
	virtual void __fastcall DoInsert(int AIndex);
	virtual void __fastcall DoDelete(int AIndex);
	virtual void __fastcall DoClear();
public:
	__fastcall TProgramEntriesModel();
	virtual __fastcall ~TProgramEntriesModel();
	virtual void __fastcall Add(TProgramEntry* AItem);
	virtual void __fastcall Delete(int AIndex);
	virtual void __fastcall Clear();
	virtual int __fastcall Find(TProgramEntry* AItem);
	virtual int __fastcall FindByDisplayName(const AnsiString& ADisplayName);
	virtual AnsiString __fastcall EnsureUniqueDisplayName(const AnsiString& ADisplayName);
	virtual void __fastcall LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual void __fastcall SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	__property TProgramEntry* Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount};
	__property int Capacity = {read=GetCapacity, write=SetCapacity};
	__property TInsertEvent OnInsert = {read=FOnInsert, write=FOnInsert};
	__property TDeleteEvent OnDelete = {read=FOnDelete, write=FOnDelete};
	__property TNotifyEvent OnClear = {read=FOnClear, write=FOnClear};
};
//---------------------------------------------------------------------------
class TProgramEntriesSelection : public TSettingsHandler
{
private:
	TProgramEntriesModel* FProgramEntriesModel;
	TComboBox* FComboBox;
	int __fastcall GetCount();
	int __fastcall GetSelectionIndex();
	void __fastcall SetSelectionIndex(int ASelectionIndex);
	__property TProgramEntriesModel* ProgramEntriesModel = {read=FProgramEntriesModel};
	__property TComboBox* ComboBox = {read=FComboBox};
protected:
public:
	__fastcall TProgramEntriesSelection(TProgramEntriesModel* AProgramEntriesModel, TComboBox* AComboBox);
	virtual __fastcall ~TProgramEntriesSelection();
	virtual void __fastcall HandleInsert(TObject* ASender, int AIndex);
	virtual void __fastcall HandleDelete(TObject* ASender, int AIndex);
	virtual void __fastcall HandleClear(TObject* ASender);
	virtual void __fastcall LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual void __fastcall SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	__property int Count = {read=GetCount};
	__property int SelectionIndex = {read=GetSelectionIndex, write=SetSelectionIndex};
};
//---------------------------------------------------------------------------
class TProgramEntriesCommands : public TCommands
{
private:
	TProgramEntriesModel* FProgramEntriesModel;
	TProgramEntriesSelection* FProgramEntriesSelection;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall ProgramDialogDisplayNameInputForAdd(TObject* ASender, const AnsiString& ADisplayName, bool& AAccept);
	void __fastcall ProgramDialogDisplayNameInputForDuplicate(TObject* ASender, const AnsiString& ADisplayName, bool& AAccept);
	void __fastcall ProgramDialogDisplayNameInputForProperties(TObject* ASender, const AnsiString& ADisplayName, bool& AAccept);
	__property TProgramEntriesModel* ProgramEntriesModel = {read=FProgramEntriesModel};
	__property TProgramEntriesSelection* ProgramEntriesSelection = {read=FProgramEntriesSelection};
protected:
public:
	__fastcall TProgramEntriesCommands(TProgramEntriesModel* AProgramEntriesModel, TProgramEntriesSelection* AProgramEntriesSelection);
	virtual __fastcall ~TProgramEntriesCommands();
	virtual void __fastcall Add();
	virtual bool __fastcall CanAdd();
	virtual void __fastcall Duplicate();
	virtual bool __fastcall CanDuplicate();
	virtual void __fastcall Delete();
	virtual bool __fastcall CanDelete();
	virtual void __fastcall Start();
	virtual bool __fastcall CanStart();
	virtual void __fastcall Explore();
	virtual bool __fastcall CanExplore();
	virtual void __fastcall ShowProperties();
	virtual bool __fastcall CanShowProperties();
};
//---------------------------------------------------------------------------
} // End namespace App_programs
using namespace App_programs;
//---------------------------------------------------------------------------
#endif

