//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_Clipboard.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// Version: 1.0.2
// Date: 2012-06-24 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
namespace Moa_clipboard {
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Initialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Uninitialize()
{
	Detach();
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Attach()
{
	if(FViewer != NULL) {
		return;
	}
	HWND LViewer = AllocateHWnd(HandleMessage);
	HWND LNextViewer = NULL;
	try {
		::SetLastError(NO_ERROR);
		LNextViewer = ::SetClipboardViewer(LViewer);
		if(::GetLastError() != NO_ERROR) {
			throw EClipboardError("The program could not add the window to the clipboard viewer chain.");
		}
	}
	catch(...) {
		DeallocateHWnd(LViewer);
		throw;
	}
	FViewer = LViewer;
	FNextViewer = LNextViewer;
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Detach()
{
	if(FViewer == NULL) {
		return;
	}
	::ChangeClipboardChain(Viewer, NextViewer);
	FNextViewer = NULL;
	DeallocateHWnd(FViewer);
	FViewer = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::HandleMessage(TMessage& AMessage)
{
	switch(AMessage.Msg) {
	case WM_DRAWCLIPBOARD:
		WMDrawClipboard(AMessage);
		break;
	case WM_CHANGECBCHAIN:
		WMChangeCbChain(AMessage);
		break;
	default:
		AMessage.Result = ::DefWindowProc(Viewer, AMessage.Msg, AMessage.WParam, AMessage.LParam);
		break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::WMDrawClipboard(TMessage& AMessage)
{
	DoChange();
	if(NextViewer != NULL) {
		AMessage.Result = ::SendMessage(NextViewer, AMessage.Msg, AMessage.WParam, AMessage.LParam);
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::WMChangeCbChain(TMessage& AMessage)
{
	if(reinterpret_cast<HWND>(AMessage.WParam) != NextViewer) {
		AMessage.Result = ::SendMessage(NextViewer, AMessage.Msg, AMessage.WParam, AMessage.LParam);
		return;
	}
	FNextViewer = reinterpret_cast<HWND>(AMessage.LParam);
	AMessage.Result = true;
}
//---------------------------------------------------------------------------
bool __fastcall TClipboardWatcher::GetEnabled()
{
	return Viewer != NULL;
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::SetEnabled(bool AEnabled)
{
	if((AEnabled ? 1 : 0) == (Enabled ? 1 : 0)) {
		return;
	}
	if(AEnabled) {
		Attach();
	}
	else {
		Detach();
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TClipboardWatcher::TClipboardWatcher() :
	TObject(),
	FViewer(NULL),
	FNextViewer(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TClipboardWatcher::~TClipboardWatcher()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
} // End namespace Moa_clipboard
//---------------------------------------------------------------------------

