//---------------------------------------------------------------------------
#ifndef Moa_DynamicArraysH
#define Moa_DynamicArraysH
//---------------------------------------------------------------------------
// Version: 1.2.0
// Date: 2012-06-24 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_dynamicarrays {
//---------------------------------------------------------------------------
class EDynamicArrayError : public Exception
{
public:
	__fastcall EDynamicArrayError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
template <typename T>
class TDynamicArrayBuilder : public TObject
{
private:
	DynamicArray<T> FBuffer;
	int FLength;
	DynamicArray<T> __fastcall GetCurrent();
	void __fastcall SetCurrent(const DynamicArray<T>& ACurrent);
	void __fastcall SetLength(int ALength);
	int __fastcall GetCapacity();
	void __fastcall SetCapacity(int ACapacity);
	__property DynamicArray<T> Buffer = {read=FBuffer};
protected:
public:
	__fastcall TDynamicArrayBuilder();
	virtual __fastcall ~TDynamicArrayBuilder();
	virtual int __fastcall Add(const T& AElement);
	virtual void __fastcall Clear();
	virtual void __fastcall Expand();
	virtual DynamicArray<T> __fastcall Finish();
	__property DynamicArray<T> Current = {read=GetCurrent, write=SetCurrent};
	__property int Length = {read=FLength, write=SetLength};
	__property int Capacity = {read=GetCapacity, write=SetCapacity};
};
//---------------------------------------------------------------------------
template <typename T>
class TDynamicArrayUtilities
{
private:
protected:
public:
	static DynamicArray<T> __fastcall FromElements(const T& AElement1)
	{
		DynamicArray<T> LArray;
		LArray.Length = 1;
		LArray[0] = AElement1;
		return LArray;
	}
	static DynamicArray<T> __fastcall FromElements(const T& AElement1, const T& AElement2)
	{
		DynamicArray<T> LArray;
		LArray.Length = 2;
		LArray[0] = AElement1;
		LArray[1] = AElement2;
		return LArray;
	}
	static DynamicArray<T> __fastcall FromElements(const T& AElement1, const T& AElement2, const T& AElement3)
	{
		DynamicArray<T> LArray;
		LArray.Length = 3;
		LArray[0] = AElement1;
		LArray[1] = AElement2;
		LArray[2] = AElement3;
		return LArray;
	}
	static DynamicArray<T> __fastcall FromElements(const T& AElement1, const T& AElement2, const T& AElement3, const T& AElement4)
	{
		DynamicArray<T> LArray;
		LArray.Length = 4;
		LArray[0] = AElement1;
		LArray[1] = AElement2;
		LArray[2] = AElement3;
		LArray[3] = AElement4;
		return LArray;
	}
	static DynamicArray<T> __fastcall FromElements(const T& AElement1, const T& AElement2, const T& AElement3, const T& AElement4, const T& AElement5)
	{
		DynamicArray<T> LArray;
		LArray.Length = 5;
		LArray[0] = AElement1;
		LArray[1] = AElement2;
		LArray[2] = AElement3;
		LArray[3] = AElement4;
		LArray[4] = AElement5;
		return LArray;
	}
	static DynamicArray<T> __fastcall FromElements(const T& AElement1, const T& AElement2, const T& AElement3, const T& AElement4, const T& AElement5, const T& AElement6)
	{
		DynamicArray<T> LArray;
		LArray.Length = 6;
		LArray[0] = AElement1;
		LArray[1] = AElement2;
		LArray[2] = AElement3;
		LArray[3] = AElement4;
		LArray[4] = AElement5;
		LArray[5] = AElement6;
		return LArray;
	}
	static DynamicArray<T> __fastcall FromElements(const T& AElement1, const T& AElement2, const T& AElement3, const T& AElement4, const T& AElement5, const T& AElement6, const T& AElement7)
	{
		DynamicArray<T> LArray;
		LArray.Length = 7;
		LArray[0] = AElement1;
		LArray[1] = AElement2;
		LArray[2] = AElement3;
		LArray[3] = AElement4;
		LArray[4] = AElement5;
		LArray[5] = AElement6;
		LArray[6] = AElement7;
		return LArray;
	}
	static DynamicArray<T> __fastcall FromElements(const T& AElement1, const T& AElement2, const T& AElement3, const T& AElement4, const T& AElement5, const T& AElement6, const T& AElement7, const T& AElement8)
	{
		DynamicArray<T> LArray;
		LArray.Length = 8;
		LArray[0] = AElement1;
		LArray[1] = AElement2;
		LArray[2] = AElement3;
		LArray[3] = AElement4;
		LArray[4] = AElement5;
		LArray[5] = AElement6;
		LArray[6] = AElement7;
		LArray[7] = AElement8;
		return LArray;
	}
	static DynamicArray<T> __fastcall FromElements(const T& AElement1, const T& AElement2, const T& AElement3, const T& AElement4, const T& AElement5, const T& AElement6, const T& AElement7, const T& AElement8, const T& AElement9)
	{
		DynamicArray<T> LArray;
		LArray.Length = 9;
		LArray[0] = AElement1;
		LArray[1] = AElement2;
		LArray[2] = AElement3;
		LArray[3] = AElement4;
		LArray[4] = AElement5;
		LArray[5] = AElement6;
		LArray[6] = AElement7;
		LArray[7] = AElement8;
		LArray[8] = AElement9;
		return LArray;
	}
	static DynamicArray<T> __fastcall ReverseOrder(const DynamicArray<T>& AArray)
	{
		DynamicArray<T> LArray;
		LArray.Length = AArray.Length;
		for(int LI = 0; LI < LArray.Length; ++LI) {
			LArray[LI] = AArray[LArray.High - LI];
		}
		return LArray;
	}
};
//---------------------------------------------------------------------------
class TAnsiStringDynamicArrayUtilities : TDynamicArrayUtilities<AnsiString>
{
private:
protected:
public:
	static TStringDynArray __fastcall FromList(TStrings* AList);
	static void __fastcall ToList(const TStringDynArray& AArray, TStrings* AList);
	static TStringDynArray __fastcall FromText(const AnsiString& AText);
	static AnsiString __fastcall ToText(const TStringDynArray& AArray);
	static TStringDynArray __fastcall FromCommaText(const AnsiString& ACommaText);
	static AnsiString __fastcall ToCommaText(const TStringDynArray& AArray);
	static TStringDynArray __fastcall RemoveBlankLines(const TStringDynArray& AArray);
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
template <typename T>
DynamicArray<T> __fastcall TDynamicArrayBuilder<T>::GetCurrent()
{
	return Buffer.CopyRange(0, Length);
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TDynamicArrayBuilder<T>::SetCurrent(const DynamicArray<T>& ACurrent)
{
	FBuffer = ACurrent;
	FLength = Buffer.Length;
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TDynamicArrayBuilder<T>::SetLength(int ALength)
{
	if(ALength < 0) {
		throw EDynamicArrayError("The program cannot set the length.");
	}
	if(ALength == Length) {
		return;
	}
	if(ALength > Capacity) {
		Capacity = ALength;
	}
	FLength = ALength;
}
//---------------------------------------------------------------------------
template <typename T>
int __fastcall TDynamicArrayBuilder<T>::GetCapacity()
{
	return Buffer.Length;
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TDynamicArrayBuilder<T>::SetCapacity(int ACapacity)
{
	if(ACapacity < Length) {
		throw EDynamicArrayError("The program cannot set the capacity.");
	}
	FBuffer.Length = ACapacity;
}
//---------------------------------------------------------------------------
template <typename T>
__fastcall TDynamicArrayBuilder<T>::TDynamicArrayBuilder() :
	TObject(),
	FBuffer(),
	FLength(0)
{
	// Do nothing
}
//---------------------------------------------------------------------------
template <typename T>
__fastcall TDynamicArrayBuilder<T>::~TDynamicArrayBuilder()
{
	// Do nothing
}
//---------------------------------------------------------------------------
template <typename T>
int __fastcall TDynamicArrayBuilder<T>::Add(const T& AElement)
{
	Expand();
	int LIndex = Length;
	FBuffer[LIndex] = AElement;
	++FLength;
	return LIndex;
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TDynamicArrayBuilder<T>::Clear()
{
	if(Length == 0) {
		return;
	}
	FLength = 0;
	FBuffer.Length = 0;
}
//---------------------------------------------------------------------------
template <typename T>
void __fastcall TDynamicArrayBuilder<T>::Expand()
{
	if(Capacity > Length) {
		return;
	}
	if(Capacity < 8) {
		FBuffer.Length = 8;
	}
	else {
		FBuffer.Length *= 1.5;
	}
}
//---------------------------------------------------------------------------
template <typename T>
DynamicArray<T> __fastcall TDynamicArrayBuilder<T>::Finish()
{
	DynamicArray<T> LCurrent = Current;
	Clear();
	return LCurrent;
}
//---------------------------------------------------------------------------
} // End of namespace Moa_dynamicarrays
using namespace Moa_dynamicarrays;
//---------------------------------------------------------------------------
#endif

