//---------------------------------------------------------------------------
// Version: 1.2.0
// Date: 2012-06-02 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_Files.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_files {
//---------------------------------------------------------------------------
bool __fastcall TFileUtilities::Exists(const AnsiString& APath)
{
	return ::GetFileAttributes(APath.c_str()) != 0xFFFFFFFF;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileUtilities::GetShortPath(const AnsiString& APath)
{
	if(TPathUtilities::IsPattern(APath)) {
		throw EFileError("The program could not get the short path of the file.");
	}
	DWORD LBufferSize = ::GetShortPathName(APath.c_str(), NULL, 0);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the short path of the file.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	DWORD LDataSize = ::GetShortPathName(APath.c_str(), LBuffer, LBufferSize);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the short path of the file.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileUtilities::GetLongPath(const AnsiString& APath)
{
	if(TPathUtilities::IsPattern(APath)) {
		throw EFileError("The program could not get the long path of the file.");
	}
	DWORD LBufferSize = ::GetLongPathName(APath.c_str(), NULL, 0);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the long path of the file.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	DWORD LDataSize = ::GetLongPathName(APath.c_str(), LBuffer, LBufferSize);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the long path of the file.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------
DWORDLONG __fastcall TFileUtilities::GetSize(const AnsiString& APath)
{
	WIN32_FIND_DATA LData = GetData(APath);
	ULARGE_INTEGER LSize = {0};
	LSize.LowPart = LData.nFileSizeLow;
	LSize.HighPart = LData.nFileSizeHigh;
	return LSize.QuadPart;
}
//---------------------------------------------------------------------------
DWORD __fastcall TFileUtilities::GetAttributes(const AnsiString& APath)
{
	DWORD LAttributes = ::GetFileAttributes(APath.c_str());
	if(LAttributes == 0xFFFFFFFF) {
		throw EFileError("The program could not get the attributes of the file.");
	}
	return LAttributes;
}
//---------------------------------------------------------------------------
void __fastcall TFileUtilities::SetAttributes(const AnsiString& APath, DWORD AAttributes)
{
	if(!::SetFileAttributes(APath.c_str(), AAttributes)) {
		throw EFileError("The program could not set the attributes of the file.");
	}
}
//---------------------------------------------------------------------------
FILETIME __fastcall TFileUtilities::GetCreationTime(const AnsiString& APath)
{
	FILETIME LTime = {0};
	GetTimes(APath, &LTime, NULL, NULL);
	return LTime;
}
//---------------------------------------------------------------------------
void __fastcall TFileUtilities::SetCreationTime(const AnsiString& APath, const FILETIME& ATime)
{
	SetTimes(APath, &ATime, NULL, NULL);
}
//---------------------------------------------------------------------------
FILETIME __fastcall TFileUtilities::GetLastWriteTime(const AnsiString& APath)
{
	FILETIME LTime = {0};
	GetTimes(APath, NULL, &LTime, NULL);
	return LTime;
}
//---------------------------------------------------------------------------
void __fastcall TFileUtilities::SetLastWriteTime(const AnsiString& APath, const FILETIME& ATime)
{
	SetTimes(APath, NULL, &ATime, NULL);
}
//---------------------------------------------------------------------------
FILETIME __fastcall TFileUtilities::GetLastAccessTime(const AnsiString& APath)
{
	FILETIME LTime = {0};
	GetTimes(APath, NULL, NULL, &LTime);
	return LTime;
}
//---------------------------------------------------------------------------
void __fastcall TFileUtilities::SetLastAccessTime(const AnsiString& APath, const FILETIME& ATime)
{
	SetTimes(APath, NULL, NULL, &ATime);
}
//---------------------------------------------------------------------------
void __fastcall TFileUtilities::GetTimes(const AnsiString& APath, FILETIME* ACreationTime, FILETIME* ALastWriteTime, FILETIME* ALastAccessTime)
{
	WIN32_FIND_DATA LData = GetData(APath);
	if(ACreationTime != NULL) {
		*ACreationTime = LData.ftCreationTime;
	}
	if(ALastWriteTime != NULL) {
		*ALastWriteTime = LData.ftLastWriteTime;
	}
	if(ALastAccessTime != NULL) {
		*ALastAccessTime = LData.ftLastAccessTime;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileUtilities::SetTimes(const AnsiString& APath, const FILETIME* ACreationTime, const FILETIME* ALastWriteTime, const FILETIME* ALastAccessTime)
{
	HANDLE LHandle = ::CreateFile(
		APath.c_str(),
		GENERIC_WRITE,
		0,
		NULL,
		OPEN_EXISTING,
		(TFileUtilities::GetAttributes(APath) & FILE_ATTRIBUTE_DIRECTORY) != 0 ?
			FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS :
			FILE_ATTRIBUTE_NORMAL,
		NULL);
	if(LHandle == INVALID_HANDLE_VALUE) {
		throw EFileError("The program could not set the times of the file.");
	}
	try {
		if(!::SetFileTime(LHandle, ACreationTime, ALastAccessTime, ALastWriteTime)) {
			throw EFileError("The program could not set the times of the file.");
		}
	}
	__finally {
		::CloseHandle(LHandle);
	}
}
//---------------------------------------------------------------------------
WIN32_FIND_DATA __fastcall TFileUtilities::GetData(const AnsiString& APath)
{
	WIN32_FIND_DATA LData = {0};
	{
		HANDLE LHandle = ::FindFirstFile(APath.c_str(), &LData);
		if(LHandle == INVALID_HANDLE_VALUE) {
			throw EFileError("The program could not find the file.");
		}
		::FindClose(LHandle);
	}
	return LData;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TFileDataUtilities::GetLongName(const WIN32_FIND_DATA& AData)
{
	return AData.cFileName;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileDataUtilities::GetShortName(const WIN32_FIND_DATA& AData)
{
	return AData.cAlternateFileName;
}
//---------------------------------------------------------------------------
DWORDLONG __fastcall TFileDataUtilities::GetSize(const WIN32_FIND_DATA& AData)
{
	ULARGE_INTEGER LInteger = {0};
	LInteger.LowPart = AData.nFileSizeLow;
	LInteger.HighPart = AData.nFileSizeHigh;
	return LInteger.QuadPart;
}
//---------------------------------------------------------------------------
FILETIME __fastcall TFileDataUtilities::GetCreationTime(const WIN32_FIND_DATA& AData)
{
	return AData.ftCreationTime;
}
//---------------------------------------------------------------------------
FILETIME __fastcall TFileDataUtilities::GetLastWriteTime(const WIN32_FIND_DATA& AData)
{
	return AData.ftLastWriteTime;
}
//---------------------------------------------------------------------------
FILETIME __fastcall TFileDataUtilities::GetLastAccessTime(const WIN32_FIND_DATA& AData)
{
	return AData.ftLastAccessTime;
}
//---------------------------------------------------------------------------
DWORD __fastcall TFileDataUtilities::GetAttributes(const WIN32_FIND_DATA& AData)
{
	return AData.dwFileAttributes;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TRegularFileUtilities::CreateTemporary()
{
	return CreateTemporary("TMP");
}
//---------------------------------------------------------------------------
AnsiString __fastcall TRegularFileUtilities::CreateTemporary(const AnsiString& APrefix)
{
	return CreateTemporary("TMP", TSpecialFiles::GetTemporaryDirectory());
}
//---------------------------------------------------------------------------
AnsiString __fastcall TRegularFileUtilities::CreateTemporary(const AnsiString& APrefix, const AnsiString& ADirectory)
{
	TAutoArray<TCHAR> LBuffer = new TCHAR[MAX_PATH];
	::ZeroMemory(LBuffer, sizeof(TCHAR) * MAX_PATH);
	if(::GetTempFileName(ADirectory.c_str(), APrefix.c_str(), 0, LBuffer) == 0) {
		throw EFileError("The program could not create a temporary file.");
	}
	return AnsiString(LBuffer);
}
//---------------------------------------------------------------------------
void __fastcall TRegularFileUtilities::Move(const AnsiString& ASource, const AnsiString& ADestination)
{
	if(!::MoveFile(ASource.c_str(), ADestination.c_str())) {
		throw EFileError("The program could not move the file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TRegularFileUtilities::Copy(const AnsiString& ASource, const AnsiString& ADestination, bool AOverwrite)
{
	if(!::CopyFile(ASource.c_str(), ADestination.c_str(), !AOverwrite)) {
		throw EFileError("The program could not copy the file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TRegularFileUtilities::Delete(const AnsiString& APath)
{
	if(!::DeleteFile(APath.c_str())) {
		throw EFileError("The program could not delete the file.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TRegularFileUtilities::Exists(const AnsiString& APath)
{
	DWORD LAttributes = ::GetFileAttributes(APath.c_str());
	return LAttributes != 0xFFFFFFFF && (LAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TDirectoryUtilities::Create(const AnsiString& APath)
{
	if(!::CreateDirectory(APath.c_str(), NULL)) {
		throw EFileError("The program could not create the directory.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TDirectoryUtilities::FullCreate(const AnsiString& APath)
{
	if(APath.Length() == 0 || (!Exists(APath) && !Sysutils::ForceDirectories(APath))) {
		throw EFileError("The program could not create the directories.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TDirectoryUtilities::Move(const AnsiString& ASource, const AnsiString& ADestination)
{
	if(!::MoveFile(ASource.c_str(), ADestination.c_str())) {
		throw EFileError("The program could not move the directory.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TDirectoryUtilities::Delete(const AnsiString& APath)
{
	if(!::RemoveDirectory(APath.c_str())) {
		throw EFileError("The program could not delete the directory.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDirectoryUtilities::Exists(const AnsiString& APath)
{
	DWORD LAttributes = ::GetFileAttributes(APath.c_str());
	return LAttributes != 0xFFFFFFFF && (LAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialFiles::GetCurrentDirectory()
{
	DWORD LBufferSize = ::GetCurrentDirectory(0, NULL);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the current directory.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	DWORD LDataSize = ::GetCurrentDirectory(LBufferSize, LBuffer);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the current directory.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------
void __fastcall TSpecialFiles::SetCurrentDirectory(const AnsiString& ADirectory)
{
	if(!::SetCurrentDirectory(ADirectory.c_str())) {
		throw EFileError("The program could not set the current directory.");
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialFiles::GetSystemDirectory()
{
	UINT LBufferSize = ::GetSystemDirectory(NULL, 0);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the system directory.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	UINT LDataSize = ::GetSystemDirectory(LBuffer, LBufferSize);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the system directory.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialFiles::GetTemporaryDirectory()
{
	DWORD LBufferSize = ::GetTempPath(0, NULL);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the temporary directory.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	DWORD LDataSize = ::GetTempPath(LBufferSize, LBuffer);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the temporary directory.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialFiles::GetWindowsDirectory()
{
	UINT LBufferSize = ::GetWindowsDirectory(NULL, 0);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the Windows directory.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	UINT LDataSize = ::GetWindowsDirectory(LBuffer, LBufferSize);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the Windows directory.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TPathUtilities::TrimLeft(const AnsiString& APath)
{
	const char* LCurrentPosition = APath.c_str();
	while(*LCurrentPosition >= 0x01 && *LCurrentPosition <= 0x20) {
		LCurrentPosition = ::CharNext(LCurrentPosition);
	}
	return LCurrentPosition;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TPathUtilities::TrimRight(const AnsiString& APath)
{
	const char* LInitialPosition = APath.c_str();
	const char* LTerminalPosition = LInitialPosition + APath.Length();
	const char* LCurrentPosition = ::CharPrev(LInitialPosition, LTerminalPosition);
	while(*LCurrentPosition >= 0x01 && *LCurrentPosition <= 0x20) {
		LTerminalPosition = LCurrentPosition;
		LCurrentPosition = ::CharPrev(LInitialPosition, LCurrentPosition);
	}
	return AnsiString(LInitialPosition, LTerminalPosition - LInitialPosition);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TPathUtilities::Trim(const AnsiString& APath)
{
	return TrimRight(TrimLeft(APath));
}
//---------------------------------------------------------------------------
bool __fastcall TPathUtilities::IsAbsolute(const AnsiString& APath)
{
	return Sysutils::ExtractFileDrive(APath).Length() > 0;
}
//---------------------------------------------------------------------------
bool __fastcall TPathUtilities::IsRelative(const AnsiString& APath)
{
	return !IsAbsolute(APath);
}
//---------------------------------------------------------------------------
bool __fastcall TPathUtilities::IsPattern(const AnsiString& APath)
{
	const char* LPosition = APath.c_str() - 1;
	const char* LTerminatorPosition = APath.c_str() + APath.Length();
	bool LTrailingByte = false;
	while(++LPosition < LTerminatorPosition && *LPosition != '\0') {
		if(LTrailingByte) {
			LTrailingByte = false;
			continue;
		}
		if(::IsDBCSLeadByte(*LPosition)) {
			++LPosition;
			LTrailingByte = true;
			continue;
		}
		switch(*LPosition) {
		case '*':
		case '?':
			return true;
		}
	}
	return false;
}
//---------------------------------------------------------------------------
void __fastcall TPathUtilities::AssureAbsolute(const AnsiString& APath)
{
	if(!IsAbsolute(APath)) {
		throw EPathError("The path is not in absolute form.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TPathUtilities::AssureRelative(const AnsiString& APath)
{
	if(!IsRelative(APath)) {
		throw EPathError("The path is not in relative form.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TPathUtilities::AssureNotPattern(const AnsiString& APath)
{
	if(IsPattern(APath)) {
		throw EPathError("The path is a pattern.");
	}
}
//---------------------------------------------------------------------------
int __fastcall TPathUtilities::FindInvalidCharacter(const AnsiString& APath)
{
	const char* LInitialPosition = APath.c_str();
	const char* LCurrentPosition = LInitialPosition;
	while(*LCurrentPosition != '\0') {
		if(*LCurrentPosition >= 1 && *LCurrentPosition <= 31) {
			return LCurrentPosition - LInitialPosition + 1;
		}
		switch(*LCurrentPosition) {
		case '<':
		case '>':
		case '\"':
		case '/':
		case '|':
		case '?':
		case '*':
			return LCurrentPosition - LInitialPosition + 1;
		}
		LCurrentPosition = ::CharNext(LCurrentPosition);
	}
	return 0;
}
//---------------------------------------------------------------------------
int __fastcall TPathUtilities::FindInvalidNameCharacter(const AnsiString& APath)
{
	const char* LInitialPosition = APath.c_str();
	const char* LCurrentPosition = LInitialPosition;
	while(*LCurrentPosition != '\0') {
		if(*LCurrentPosition >= 1 && *LCurrentPosition <= 31) {
			return LCurrentPosition - LInitialPosition + 1;
		}
		switch(*LCurrentPosition) {
		case '<':
		case '>':
		case ':':
		case '\"':
		case '/':
		case '\\':
		case '|':
		case '?':
		case '*':
			return LCurrentPosition - LInitialPosition + 1;
		}
		LCurrentPosition = ::CharNext(LCurrentPosition);
	}
	return 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool __fastcall TFileEnumerator::FindFirst()
{
	::ZeroMemory(&FData, sizeof(WIN32_FIND_DATA));
	FHandle = ::FindFirstFile(Pattern.c_str(), &FData);
	return Handle != INVALID_HANDLE_VALUE;
}
//---------------------------------------------------------------------------
bool __fastcall TFileEnumerator::FindNext()
{
	::ZeroMemory(&FData, sizeof(WIN32_FIND_DATA));
	return ::FindNextFile(Handle, &FData);
}
//---------------------------------------------------------------------------
void __fastcall TFileEnumerator::Finish()
{
	if(Handle == INVALID_HANDLE_VALUE) {
		return;
	}
	if(Handle != NULL) {
		::FindClose(FHandle);
	}
	FHandle = INVALID_HANDLE_VALUE;
}
//---------------------------------------------------------------------------
WIN32_FIND_DATA __fastcall TFileEnumerator::GetCurrent()
{
	return FData;
}
//---------------------------------------------------------------------------
__fastcall TFileEnumerator::TFileEnumerator(const AnsiString& APattern) :
	TObject(),
	FPattern(APattern),
	FHandle(NULL),
	FData()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFileEnumerator::~TFileEnumerator()
{
	Finish();
}
//---------------------------------------------------------------------------
bool __fastcall TFileEnumerator::MoveNext()
{
	if(Handle == NULL) {
		return FindFirst();
	}
	else {
		if(FindNext()) {
			return true;
		}
		else {
			Finish();
			return false;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileEnumerator::Reset()
{
	Finish();
	FHandle = NULL;
}
//---------------------------------------------------------------------------
} // End namespace Moa_files
//---------------------------------------------------------------------------

