//---------------------------------------------------------------------------
// Version: 1.3.0
// Date: 2012-06-21 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_ShellFiles.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_shellfiles {
//---------------------------------------------------------------------------
AnsiString __fastcall TFileTypeCache::TEntry::Determine(const AnsiString& AFile, bool AAsDirectory)
{
	SHFILEINFO LFileInfo = {0};
	if(::SHGetFileInfo(
		AFile.c_str(),
		AAsDirectory ? FILE_ATTRIBUTE_DIRECTORY : FILE_ATTRIBUTE_NORMAL,
		&LFileInfo,
		sizeof(SHFILEINFO),
		SHGFI_TYPENAME | SHGFI_USEFILEATTRIBUTES) == 0) {
		throw EShellFileError("The program could not get the file type.");
	}
	return LFileInfo.szTypeName;
}
//---------------------------------------------------------------------------
__fastcall TFileTypeCache::TEntry::TEntry() :
	TObject(),
	FDescription(Determine(GetCurrentDir(), true))
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFileTypeCache::TEntry::TEntry(const AnsiString& AFile) :
	TObject(),
	FDescription(Determine(AFile, false))
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFileTypeCache::TEntry::~TEntry()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TFileTypeCache::Initialize()
{
	try {
		FExtensions = new TStringList();
		FExtensions->Sorted = true;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileTypeCache::Uninitialize()
{
	delete FDirectory;
	FDirectory = NULL;

	if(FExtensions != NULL) {
		for(int LI = 0; LI < FExtensions->Count; ++LI) {
			delete FExtensions->Objects[LI];
		}
		delete FExtensions;
		FExtensions = NULL;
	}
}
//---------------------------------------------------------------------------
__fastcall TFileTypeCache::TFileTypeCache() :
	TObject(),
	FExtensions(NULL),
	FDirectory(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TFileTypeCache::~TFileTypeCache()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileTypeCache::Determine(const AnsiString& AFile)
{
	DWORD LFileAttributes = ::GetFileAttributes(AFile.c_str());
	if(LFileAttributes == 0xFFFFFFFF) {
		throw EShellFileError("The program cannot determine the file type.");
	}
	if((LFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0) {
		return DetermineAsDirectory();
	}
	else {
		return DetermineAsRegularFile(AFile);
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileTypeCache::DetermineAsRegularFile(const AnsiString& AFile)
{
	AnsiString LFile = ExpandFileName(AFile);
	AnsiString LExtension = AnsiUpperCaseFileName(ExtractFileExt(LFile));
	int LIndex;
	if(Extensions->Find(LExtension, LIndex)) {
		return static_cast<TEntry*>(Extensions->Objects[LIndex])->Description;
	}
	TAutoPointer<TEntry> LEntry = new TEntry(LFile);
	Extensions->AddObject(LExtension, LEntry);
	return LEntry.Release()->Description;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileTypeCache::DetermineAsDirectory()
{
	if(Directory == NULL) {
		FDirectory = new TEntry();
	}
	return Directory->Description;
}
//---------------------------------------------------------------------------
void __fastcall TFileTypeCache::Clear()
{
	delete FDirectory;
	FDirectory = NULL;

	for(int LI = 0; LI < Extensions->Count; ++LI) {
		delete Extensions->Objects[LI];
	}
	Extensions->Clear();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialShellFiles::Determine(UINT AId)
{
	TAutoArray<TCHAR> LBuffer = new TCHAR[MAX_PATH];
	::ZeroMemory(LBuffer, sizeof(TCHAR) * MAX_PATH);
	if(!SHGetSpecialFolderPath(NULL, LBuffer, AId, false)) {
		throw EShellFileError("The program could not get the path of the special file.");
	}
	return AnsiString(LBuffer);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialShellFiles::GetApplicationData() 
{
	return Determine(CSIDL_APPDATA);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialShellFiles::GetDesktop() 
{
	return Determine(CSIDL_DESKTOPDIRECTORY);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialShellFiles::GetDocuments() 
{
	return Determine(CSIDL_PERSONAL);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialShellFiles::GetFavorites() 
{
	return Determine(CSIDL_FAVORITES);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialShellFiles::GetProgramFiles() 
{
	return Determine(CSIDL_PROGRAM_FILES);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialShellFiles::GetPrograms() 
{
	return Determine(CSIDL_PROGRAMS);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialShellFiles::GetStartMenu()
{
	return Determine(CSIDL_STARTMENU);
}
//---------------------------------------------------------------------------
} // namespace Moa_shellfiles
//---------------------------------------------------------------------------

