//---------------------------------------------------------------------------
// Version: 1.1.0
// Date: 2012-06-19 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_Tasks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_tasks {
//---------------------------------------------------------------------------
class TTaskThread : public TThread
{
private:
	TNotifyEvent FOnExecute;
protected:
	virtual void __fastcall Execute();
	virtual void __fastcall DoExecute();
public:
	__fastcall TTaskThread(bool ACreateSuspended);
	virtual __fastcall ~TTaskThread();
	using TThread::Synchronize;
	__property TNotifyEvent OnExecute = {read=FOnExecute, write=FOnExecute};
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TTaskThread::Execute()
{
	DoExecute();
}
//---------------------------------------------------------------------------
void __fastcall TTaskThread::DoExecute()
{
	if(OnExecute != NULL) {
		OnExecute(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TTaskThread::TTaskThread(bool ACreateSuspended) :
	TThread(ACreateSuspended),
	FOnExecute(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTaskThread::~TTaskThread()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedDoCancel(void* AData)
{
	OnCancel(this);
}
//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedDoTerminate(void* AData)
{
	OnTerminate(this);
}
//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedDoProgress(void* AData)
{
	TOnProgressArguments& LArguments = *static_cast<TOnProgressArguments*>(AData);
	OnProgress(this, LArguments.Current, LArguments.All);
}
//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedDoException(void* AData)
{
	TOnExceptionArguments& LArguments = *static_cast<TOnExceptionArguments*>(AData);
	OnException(this, LArguments.ExceptionObject);
}
//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedSynchronize()
{
	try {
		SynchronizeArguments.Method(SynchronizeArguments.Data);
	}
	catch(EAbort& LException) {
		FSynchronizeArguments.ExceptionCaught = true;
		FSynchronizeArguments.Aborted = true;
		FSynchronizeArguments.ExceptionMessage = LException.Message;
	}
	catch(Exception& LException) {
		FSynchronizeArguments.ExceptionCaught = true;
		FSynchronizeArguments.ExceptionMessage = LException.Message;
	}
	catch(...) {
		FSynchronizeArguments.ExceptionCaught = true;
		FSynchronizeArguments.ExceptionMessage = "An unknown exception occured.";
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::ThreadExecute(TObject* ASender)
{
	try {
		try {
			try {
				Execute();
			}
			catch(EAbort& LException) {
				// Do nothing
			}
			catch(Exception& LException) {
				throw;
			}
			catch(...) {
				throw Exception("An unknown exception occured.");
			}
		}
		catch(Exception& LException) {
			DoException(&LException);
		}
	}
	catch(...) {
		// Do nothing
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::ThreadTerminate(TObject* ASender)
{
	try {
		try {
			DoTerminate();
		}
		catch(...) {
			// Do nothing
		}
		if(FreeOnTerminate) {
			delete this;
		}
	}
	catch(...) {
		// Do nothing
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoCancel()
{
	if(OnCancel != NULL) {
		Synchronize(&SynchronizedDoCancel, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoTerminate()
{
	if(OnTerminate != NULL) {
		Synchronize(&SynchronizedDoTerminate, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoProgress(int ACurrent, int AAll)
{
	if(OnProgress != NULL) {
		TOnProgressArguments LArguments = {ACurrent, AAll};
		Synchronize(&SynchronizedDoProgress, &LArguments);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoException(Exception* AException)
{
	if(OnException != NULL) {
		TOnExceptionArguments LArguments = {AException};
		Synchronize(&SynchronizedDoException, &LArguments);
	}
}
//---------------------------------------------------------------------------
__fastcall TTask::TTask() :
	TObject(),
	FThread(NULL),
	FCanceled(false),
	FFreeOnTerminate(false),
	FSynchronizeArguments(),
	FOnCancel(NULL),
	FOnTerminate(NULL),
	FOnProgress(NULL),
	FOnException(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTask::~TTask()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TTask::Perform()
{
	TAutoPointer<TTaskThread> LThread = new TTaskThread(true);
	LThread->FreeOnTerminate = true;
	LThread->OnExecute = ThreadExecute;
	LThread->OnTerminate = ThreadTerminate;
	LThread->Resume();
	FThread = LThread.Release();
}
//---------------------------------------------------------------------------
void __fastcall TTask::Cancel()
{
	if(Canceled) {
		return;
	}
	FCanceled = true;
	DoCancel();
}
//---------------------------------------------------------------------------
void __fastcall TTask::Synchronize(TTaskSynchronizeMethod AMethod, void* AData)
{
	if(::GetCurrentThreadId() == System::MainThreadID) {
		AMethod(AData);
		return;
	}
	FSynchronizeArguments.Method = AMethod;
	FSynchronizeArguments.Data = AData;
	FSynchronizeArguments.ExceptionCaught = false;
	FSynchronizeArguments.Aborted = false;
	FSynchronizeArguments.ExceptionMessage = AnsiString();
	Thread->Synchronize(&SynchronizedSynchronize);
	if(SynchronizeArguments.ExceptionCaught) {
		if(SynchronizeArguments.Aborted) {
			throw EMainThreadAbort(SynchronizeArguments.ExceptionMessage);
		}
		else {
			throw EMainThreadException(SynchronizeArguments.ExceptionMessage);
		}
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TTaskMessageBox::SynchronizedExecute(void* AData)
{
	FResult = Application->MessageBox(
		Message.c_str(),
		Title.c_str(),
		Flags);
}
//---------------------------------------------------------------------------
__fastcall TTaskMessageBox::TTaskMessageBox(TTask* ATask, const AnsiString& AMessage) :
	FTask(),
	FMessage(AMessage),
	FFlags(MB_OK | MB_SETFOREGROUND),
	FTitle(),
	FTitled(false),
	FResult(0)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTaskMessageBox::TTaskMessageBox(TTask* ATask, const AnsiString& AMessage, int AFlags) :
	FTask(),
	FMessage(AMessage),
	FFlags(AFlags),
	FTitle(),
	FTitled(false),
	FResult(0)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTaskMessageBox::TTaskMessageBox(TTask* ATask, const AnsiString& AMessage, int AFlags, const AnsiString& ATitle) :
	FTask(),
	FMessage(AMessage),
	FFlags(AFlags),
	FTitle(ATitle),
	FTitled(false),
	FResult(0)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTaskMessageBox::~TTaskMessageBox()
{
	// Do nothing
}
//---------------------------------------------------------------------------
int __fastcall TTaskMessageBox::Execute()
{
	Task->Synchronize(SynchronizedExecute, NULL);
	return Result;
}
//---------------------------------------------------------------------------
} // End namespace Moa_tasks
//---------------------------------------------------------------------------

