//---------------------------------------------------------------------------
#ifndef Moa_TasksH
#define Moa_TasksH
//---------------------------------------------------------------------------
// Version: 1.1.0
// Date: 2012-06-19 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_tasks {
//---------------------------------------------------------------------------
class ETaskError : public Exception
{
public:
	__fastcall ETaskError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class EMainThreadException : public Exception
{
public:
	__fastcall EMainThreadException(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class EMainThreadAbort : public EAbort
{
public:
	__fastcall EMainThreadAbort(const AnsiString& AMessage) : EAbort(AMessage) {}
};
//---------------------------------------------------------------------------
typedef void __fastcall (__closure * TTaskSynchronizeMethod)(void* AData);
typedef void __fastcall (__closure * TTaskProgressEvent)(TObject* ASender, int ACurrent, int AAll);
typedef void __fastcall (__closure * TTaskExceptionEvent)(TObject* ASender, Exception* AException);
//---------------------------------------------------------------------------
class TTaskThread;
class TTask : public TObject
{
private:
	struct TShowExceptionArguments
	{
		Exception* ExceptionObject;
	};
	struct TOnProgressArguments
	{
		int Current;
		int All;
	};
	struct TOnExceptionArguments
	{
		Exception* ExceptionObject;
	};
	struct TSynchronizeArguments
	{
		TTaskSynchronizeMethod Method;
		void* Data;
		bool ExceptionCaught;
		bool Aborted;
		AnsiString ExceptionMessage;
	};
	TTaskThread* FThread;
	bool FCanceled;
	bool FFreeOnTerminate;
	TSynchronizeArguments FSynchronizeArguments;
	TNotifyEvent FOnCancel;
	TNotifyEvent FOnTerminate;
	TTaskProgressEvent FOnProgress;
	TTaskExceptionEvent FOnException;
	void __fastcall SynchronizedDoCancel(void* AData);
	void __fastcall SynchronizedDoTerminate(void* AData);
	void __fastcall SynchronizedDoProgress(void* AData);
	void __fastcall SynchronizedDoException(void* AData);
	void __fastcall SynchronizedSynchronize();
	void __fastcall ThreadExecute(TObject* ASender);
	void __fastcall ThreadTerminate(TObject* ASender);
	__property TTaskThread* Thread = {read=FThread};
	__property TSynchronizeArguments SynchronizeArguments = {read=FSynchronizeArguments};
protected:
	virtual void __fastcall Execute() = 0;
	virtual void __fastcall DoCancel();
	virtual void __fastcall DoTerminate();
	virtual void __fastcall DoProgress(int ACurrent, int AAll);
	virtual void __fastcall DoException(Exception* AException);
public:
	__fastcall TTask();
	virtual __fastcall ~TTask();
	virtual void __fastcall Perform();
	virtual void __fastcall Cancel();
	virtual void __fastcall Synchronize(TTaskSynchronizeMethod AMethod, void* AData);
	__property bool Canceled = {read=FCanceled, write=FCanceled};
	__property bool FreeOnTerminate = {read=FFreeOnTerminate, write=FFreeOnTerminate};
	__property TNotifyEvent OnCancel = {read=FOnCancel, write=FOnCancel};
	__property TNotifyEvent OnTerminate = {read=FOnTerminate, write=FOnTerminate};
	__property TTaskProgressEvent OnProgress = {read=FOnProgress, write=FOnProgress};
	__property TTaskExceptionEvent OnException = {read=FOnException, write=FOnException};
};
//---------------------------------------------------------------------------
class TTaskMessageBox
{
private:
	TTask* FTask;
	AnsiString FMessage;
	int FFlags;
	AnsiString FTitle;
	bool FTitled;
	int FResult;
	void __fastcall SynchronizedExecute(void* AData);
	__property TTask* Task = {read=FTask};
	__property AnsiString Message = {read=FMessage};
	__property int Flags = {read=FFlags};
	__property AnsiString Title = {read=FTitle};
	__property bool Titled = {read=FTitled};
	__property int Result = {read=FResult};
protected:
public:
	__fastcall TTaskMessageBox(TTask* ATask, const AnsiString& AMessage);
	__fastcall TTaskMessageBox(TTask* ATask, const AnsiString& AMessage, int AFlags);
	__fastcall TTaskMessageBox(TTask* ATask, const AnsiString& AMessage, int AFlags, const AnsiString& ATitle);
	virtual __fastcall ~TTaskMessageBox();
	virtual int __fastcall Execute();
};
//---------------------------------------------------------------------------
} // End namespace Moa_tasks
using namespace Moa_tasks;
//---------------------------------------------------------------------------
#endif

